import numpy as np
import pandas as pd
from hbshare.fe.XZ import db_engine as dbeng
from hbshare.fe.XZ import functionality


util=functionality.Untils()
hbdb=dbeng.HBDB()
localdb = dbeng.PrvFunDB().engine

def save_mutual_core_pool2locabdb(asofdate):

    data=pd.read_excel(r"C:\Users\xuhuai.zhe\Desktop\核心池列表-20220629_风格.xlsx")

    data=data[['基金代码','基金名称']]
    data['基金经理']=''
    data['asofdate']=asofdate

    data['基金代码']=data['基金代码'].astype(str).str[0:6]

    data.to_sql('core_pool_history',
                con=localdb,index=False,if_exists='append')

def save_wenjian_pool2locabdb(asofdate,factor_name):

    pool_size = 10

    raw_df=pd.read_sql("select * from factor_year_rank_quantile",
                       con=localdb)


    localdb.execute("delete from conservative_pool_{1}_history where asofdate='{0}'".format(asofdate,factor_name.split('%')[0]))

    raw_df=raw_df[(raw_df['date']==asofdate)&(raw_df['count']==17)].sort_values(factor_name
                                               ,ascending=False)[0:pool_size][['jjdm',factor_name]]
    raw_df['asofdate']=asofdate

    raw_df.to_sql('conservative_pool_{0}_history'.format(factor_name.split('%')[0]),con=localdb,index=False,if_exists='append')

def save_wenjian_pool2locabdb(asofdate,factor_name):

    pool_size = 10

    raw_df=pd.read_sql("select * from factor_year_rank_quantile",
                       con=localdb)


    localdb.execute("delete from conservative_pool_{1}_history where asofdate='{0}'".format(asofdate,factor_name.split('%')[0]))

    raw_df=raw_df[(raw_df['date']==asofdate)&(raw_df['count']==17)].sort_values(factor_name
                                               ,ascending=False)[0:pool_size][['jjdm',factor_name]]
    raw_df['asofdate']=asofdate

    raw_df.to_sql('conservative_pool_{0}_history'.format(factor_name.split('%')[0]),con=localdb,index=False,if_exists='append')

def save_momentum_pool2locabdb(asofdate):

    pool_size = 20
    factor_name='year_logret'
    fre='Q'
    raw_df=pd.read_sql("select * from factor_year_ret where  {0} is not null "
                       .format(factor_name)
                       ,con=localdb)


    localdb.execute("delete from momentum_pool_history where asofdate='{}'".format(asofdate))

    raw_df=raw_df[(raw_df['date']==asofdate)].sort_values(factor_name
                                               ,ascending=False)[0:pool_size][['jjdm',factor_name]]
    raw_df['asofdate']=asofdate

    raw_df.to_sql('momentum_pool_history',con=localdb,index=False,if_exists='append')

def save_info_ratio_pool2locabdb(asofdate):

    pool_size = 50
    factor_name='info_ratio'
    fre='Q'
    raw_df=pd.read_sql("select * from factor_infor_ratio where zblb='2101'",
                       con=localdb)


    localdb.execute("delete from info_ratio_pool_history where asofdate='{}'".format(asofdate))

    raw_df=raw_df[(raw_df['date']==int(asofdate))].sort_values(factor_name
                                               ,ascending=False)[0:pool_size][['jjdm',factor_name]]
    raw_df['asofdate']=asofdate

    raw_df.to_sql('info_ratio_pool_history',con=localdb,index=False,if_exists='append')

if __name__ == '__main__':


    # save_info_ratio_pool2locabdb('20211227')
    # save_info_ratio_pool2locabdb('20220328')
    # save_info_ratio_pool2locabdb('20220627')

    #save_momentum_pool2locabdb('20220630')
    #
    save_wenjian_pool2locabdb('20220630','25%')
    save_wenjian_pool2locabdb('20220331','25%')
    save_wenjian_pool2locabdb('20211231','25%')

    save_wenjian_pool2locabdb('20220630','10%')
    save_wenjian_pool2locabdb('20220331','10%')
    save_wenjian_pool2locabdb('20211231','10%')

    #save_mutual_core_pool2locabdb('202206')

    print('Done')


from hbshare.fe.XZ import project
from hbshare.fe.XZ import functionality
import pandas as pd


projectC = project.Projects()
plot = functionality.Plot(fig_width=400, fig_height=600)

index_list = ['HSI001', '881001', 'W00083', 'ND100']
bond_list =['003328', '003847', '700006', '004706', '160618', '003474']
assets_lists = index_list + bond_list
risk_aversion_edited=0.6201726167388938
selected_date='20220430'
selected_v='20220430_v1'
selected_date2=selected_date
selected_v2=selected_v
confidence=0.05
confidence2=confidence

ub=[0.15, 0.8, 0.15, 0.5]
ub2=[0.2, 0.2, 0.2, 0.2, 0.2, 0.2]
lb=[-0.05, -0.4, -0.05, -0.05]
lb2=[-0.1, -0.1, -0.1, -0.1, -0.1, -0.1]


columns = ['股票权重', '债券权重', '风险厌恶'] + assets_lists

draw_df = pd.DataFrame()

n = 1
while n < 21:
    Xs = 0.05 * n
    Xb = 1 - Xs
    risk_aversion_new = risk_aversion_edited / Xs + (2 - risk_aversion_edited)

    w_s = projectC.bl_model(end_date=selected_date,
                            asset_list=index_list,
                            version=selected_v, tau=confidence, ub=ub, lb=lb, asset_type='index',
                            risk_aversion=risk_aversion_new)
    w_s['Weight'] = w_s['Weight'] * Xs
    w_s['Weight'] = ['{:.2%}'.format(x) for x in w_s['Weight']]

    w_b = projectC.bl_model(end_date=selected_date2,
                            asset_list=bond_list,
                            version=selected_v2, tau=confidence2, ub=ub2, lb=lb2, asset_type='public_fund',
                            risk_aversion=risk_aversion_new)

    w_b['Weight'] = w_b['Weight'] * Xb
    w_b['Weight'] = ['{:.2%}'.format(x) for x in w_b['Weight']]

    Xs = '{:.2%}'.format(Xs)
    Xb = '{:.2%}'.format(Xb)
    risk_aversion_new = '{:.4f}'.format(risk_aversion_new)
    tempdf = pd.DataFrame()
    tempdf['Asset'] = ['stock_weight', 'bond_weight', 'risk_aversion']
    tempdf['Weight'] = [Xs, Xb, risk_aversion_new]

    inputdf = pd.concat([tempdf, w_s, w_b], axis=0)
    inputdf = inputdf['Weight'].to_frame().T
    draw_df = pd.concat([draw_df, inputdf], axis=0)

    n += 1

draw_df.columns=columns
draw_df.to_csv('blresult_{}.csv'.format(selected_date),encoding='gbk')


"""
雪球产品测算
"""
import hbshare as hbs
import pandas as pd
from hbshare.fe.common.util.data_loader import get_trading_day_list


def snow_ball_simulation(start_date, end_date, duration, knock_in_price, knock_out_price):
    """
    start_date: 回测起始时间
    end_date: 回测结束时间
    duration: 合约期限
    knock_in_price: 敲入价格
    knock_out_price: 敲出价格
    """
    sql_script = "SELECT JYRQ as TRADEDATE, ZQMC as INDEXNAME, SPJG as TCLOSE from funddb.ZSJY WHERE ZQDM = '{}' " \
                 "and JYRQ >= {} and JYRQ <= {}".format('000905', start_date, end_date)
    index_df = pd.DataFrame(hbs.db_data_query('readonly', sql_script, page_size=5000)['data'])
    index_series = index_df.set_index('TRADEDATE')['TCLOSE']
    # 每个月底发一只雪球产品
    month_end_list = get_trading_day_list(start_date, end_date, frequency="month")

    res_df = pd.DataFrame(index=month_end_list[:-duration], columns=['c1', 'c2', 'c3', 'ko_time', 'lose_ratio'])

    for i in range(duration, len(month_end_list)):
        period_start = month_end_list[i - duration]
        period_end = month_end_list[i]

        period_data = index_series.loc[period_start: period_end]
        period_data = period_data / period_data.iloc[0]
        ko_date_list = [x for x in month_end_list if period_start < x <= period_end]

        if period_data.loc[ko_date_list].gt(knock_out_price).sum() > 0:  # 情形1：敲出获利
            tmp = period_data.loc[ko_date_list].to_frame('nav')
            tmp.loc[tmp['nav'] > knock_out_price, 'sign'] = 1
            ko_time = tmp.index.to_list().index(tmp['sign'].first_valid_index()) + 1
            res_df.loc[period_start, 'c1'] = 1
            res_df.loc[period_start, 'ko_time'] = ko_time
        elif period_data[1:].min() >= knock_in_price:  # 情形2：既未敲出也未敲入
            res_df.loc[period_start, 'c2'] = 1
        else:
            res_df.loc[period_start, 'c3'] = 1
            lose_ratio = \
                max(0, (period_data.loc[period_start] - period_data.loc[period_end]) / period_data.loc[period_start])
            res_df.loc[period_start, 'lose_ratio'] = lose_ratio * (-1)

    print(res_df[res_df.columns[:3]].sum() / res_df.shape[0])
    print(res_df['ko_time'].mean())
    print(res_df['lose_ratio'].mean())

if __name__ == '__main__':
    snow_ball_simulation('20100101', '20220531', 12, 0.8, 1.03)



#!/usr/bin/python
#coding:utf-8

"""
@author: Meng.lv
@contact: meng.lv@howbuy.com
@software: PyCharm
@file: data_query.py
@time: 2021/1/20 8:44
"""
import time
import hbshare as hbs
from hbshare.db.fund import cons as ct


def db_data_query(db, sql, page_size=1000, page_num=1, is_pagination=True, retry_count=3, pause=0.01, timeout=300):
    """
        通用DB数据查询接口
    :param db: 数据库名（readonly(ORACLE库), alluser(MYSQL库), smpp(SMPP库)）
    :param sql: 执行脚本
    :param page_size: 每页大小
    :param page_num: 当前页码
    :param is_pagination: 是否分页
    :param retry_count: 重试次数
    :param pause: 重试间隔时间
    :param timeout: 超时时间 (0,600]
    :return:
    """
    api = hbs.hb_api(timeout=timeout)
    for _ in range(retry_count):
        time.sleep(pause)
        #ct._write_console()
        url = "http://fpc-data.intelnal.howbuy.com/data/query/commonapi?dataTrack=%s" % (time.time())
        post_body = {
                "database": db,
                "sql": sql,
                "ifByPage": is_pagination,
                "pageNum": page_num,
                "pageSize": page_size
        }
        data = api.query(url, 'post', post_body)
        success = data['success']
        if success != 1:
            status = str(data['returnCode'])+":"+str(data['returnMsg'])
            raise ValueError(status)
        # data_frame = pd.DataFrame(data['data'])
        # pagination = {
        #     "pageNum": page_num,
        #     "pageSize": page_size,
        #     "total": int(data['total']),
        #     "pages": int(data['pages'])
        # }
        return data
    raise IOError(ct.NETWORK_URL_ERROR_MSG)




# -*- coding: utf-8 -*-

import pandas as pd
from hbshare.fe.common.util.exception import InputParameterError
from hbshare.fe.common.util.logger import logger


def verify_trading_day(trading_day_list):
    try:
        pd.DatetimeIndex(trading_day_list)
    except Exception as e:
        msg = "trading day is of bad format: %s" % e.message
        logger.error(msg)
        raise InputParameterError(msg)


def verify_type(var, var_name, var_type, trading_day=None):
    if not isinstance(var, var_type):
        msg = '%s, %s is not a %s, check your input' % (
            trading_day, var_name, str(var_type)) if trading_day is not None else \
            '%s is not a %s, check your input' % (var_name, str(var_type))
        logger.error(msg)
        raise InputParameterError(msg)


def verify_df_not_null(var, var_name, trading_day=None):
    if not var.notnull().all().all():
        msg = 'Date: %s, %s has NaN value, check your input' % (trading_day, var_name) if trading_day is not None else\
            '%s has NaN value, check your input' % var_name
        logger.error(msg)
        raise InputParameterError(msg)


def __verify_risk_model_one_day(trading_day,
                                risk_model_one_day,
                                sub_factors=None):
    if not isinstance(risk_model_one_day, dict):
        msg = '%s risk_model is not a dict, check your input' % trading_day
        logger.error(msg)
        raise InputParameterError(msg)
    verify_type(risk_model_one_day, 'risk_model_one_day', dict)
    exposure_df = risk_model_one_day.get('exposure')
    factor_covariance_df = risk_model_one_day.get('factor_covariance')
    specific_risk_series = risk_model_one_day.get('specific_risk')
    if sub_factors is None:
        verify_type(exposure_df, 'exposure_df', pd.DataFrame, trading_day)
        verify_type(factor_covariance_df, 'factor_covariance_df', pd.DataFrame, trading_day)
        verify_type(specific_risk_series, 'specific_risk_series', pd.Series, trading_day)
        verify_df_not_null(exposure_df, 'exposure_df', trading_day)
        verify_df_not_null(factor_covariance_df, 'factor_covariance_df', trading_day)
        verify_df_not_null(specific_risk_series, 'specific_risk_series', trading_day)
    else:
        verify_type(specific_risk_series, 'specific_risk_series', pd.Series, trading_day)


def verify_risk_model_hist(risk_model_dict, trading_day_list, sub_factors=None):
    verify_type(risk_model_dict, 'risk_model_dict', dict)

    if 'data' not in risk_model_dict.keys() or 'schema' not in risk_model_dict.keys():
        msg = 'missing key data or schema in risk_model'
        logger.error(msg)
        raise InputParameterError(msg)

    if 'industry_field' not in risk_model_dict['schema']:
        msg = 'missing key industry_field in the schema of risk_model'
        logger.error(msg)
        raise InputParameterError(msg)

    if 'style_field' not in risk_model_dict['schema']:
        msg = 'missing key style_field in the schema of risk_model'
        logger.error(msg)
        raise InputParameterError(msg)

    for trading_day in trading_day_list:
        risk_model_one_day = risk_model_dict['data'].get(trading_day)
        if risk_model_one_day is not None:
            __verify_risk_model_one_day(
                trading_day, risk_model_one_day, sub_factors=sub_factors)
        else:
            msg = 'missing risk_model data on %s' % trading_day
            logger.error(msg)
            raise InputParameterError(msg)


def verify_multiple_type(var, var_name, multiple_type, trading_day=None):
    if not isinstance(var, multiple_type):
        msg = '%s, %s is not a %s, check your input' % (
            trading_day, var_name, str(multiple_type)) if trading_day is not None else\
            '%s is not a %s, check your input' % var_name
        logger.error(msg)
        raise InputParameterError(msg)


def verify_simulation_day(simulation_date_list, trading_day_list):
    if simulation_date_list is None or len(simulation_date_list) == 0:
        msg = 'the simulation_date_list is empty'
        logger.error(msg)
        raise InputParameterError(msg)
    elif len(trading_day_list) != 0:
        if simulation_date_list[-1] < trading_day_list[-1] or simulation_date_list[0] > trading_day_list[0]:
            msg = 'the simulation_date_list does not cover the trading_day_list'
            logger.error(msg)
            raise InputParameterError(msg)
    else:
        msg = 'warning trading day is empty'
        logger.warning(msg)
        raise Warning(msg)


def verify_simple_data_duplicate(var, var_name, trading_day=None):
    if len(var) != len(set(var)):
        msg = "Date: %s, %s has duplicated data, check your input" % (trading_day, var_name) if \
            trading_day is not None else "%s has duplicated data, check your input" % var_name
        logger.error(msg)
        raise InputParameterError(message=msg)


def verify_container_fully_inclusion(container, target, container_name, target_name, trading_day=None, is_error=True):
    missing_inclusion_list = list(set(target) - set(container))
    missing_inclusion_list = [str(x) for x in missing_inclusion_list]
    if len(missing_inclusion_list) > 0:
        msg = "Date: %s, %s in %s are not included in %s" % (
            trading_day, ','.join(missing_inclusion_list), target_name, container_name)
        if is_error:
            logger.error(msg)
            raise InputParameterError(message=msg)
        else:
            logger.warning(msg)
            return msg



import  numpy as np
import pandas as pd
from hbshare.fe.XZ import  functionality
from hbshare.fe.XZ import db_engine as dbeng
import datetime

util=functionality.Untils()
hbdb=dbeng.HBDB()
localdb=dbeng.PrvFunDB().engine

def get_jjnav_bydate(jjdm_list,date_list):

    sql="select jjdm,jzrq,fqdwjz from st_fund.t_st_gm_rhb where jzrq in ({0}) and jjdm in ({1})"\
        .format(util.list_sql_condition(date_list),
                util.list_sql_condition(jjdm_list))

    jj_nav=hbdb.db2df(sql,db='funduser')

    return  jj_nav

def create_stock_shift_ratio():

    hsl=pd.read_excel(r"E:\GitFolder\hbshare\fe\factor_analysis\换手率.xlsx")
    date_col=hsl.columns.tolist()
    date_col.remove('证券简称')
    date_col.remove('证券代码')
    new_date_col=[]
    date_map=dict()
    hsl['jjdm'] = hsl['证券代码'].str[0:6]
    for i in range(len(date_col)):
        new_date_col.append(date_col[i].replace('H1','0630').replace(' ','').replace('H2', '1231'))

    date_map = dict(zip(date_col,new_date_col))

    outputdf=pd.DataFrame()
    tempdf=pd.DataFrame()
    tempdf['jjdm']=util.get_mutual_stock_funds('20211231')
    tempdf=pd.merge(tempdf,hsl,how='left',on='jjdm')

    for date in date_col:
        tempdf2=tempdf[['jjdm',date]]
        tempdf2['date']=date_map[date]
        tempdf2.rename(columns={date:'hsl'},inplace=True)
        outputdf=pd.concat([outputdf,tempdf2],axis=0)

    outputdf=outputdf[outputdf['hsl'].notnull()]

    outputdf.to_sql('factor_hsl',con=localdb,index=False,if_exists='append')

def read_factors(table_name):

    sql="select * from {}".format(table_name)
    df=pd.read_sql(sql,con=localdb)

    return  df

def bhar(arr):
    return 100 * (np.power(np.cumprod((arr + 100) / 100).tolist()[-1], 1 / len(arr)) - 1)

def get_quarter_ret(arr):

    return  np.cumprod(arr/100+1).tolist()[-1]-1

def hsl_rank2db(dir):

    #get the hsl raw data first
    # sql="select * from factor_hsl  "
    # df=pd.read_sql(sql,con=localdb)
    df=pd.read_csv(dir)
    df['hsl_rank']=df.groupby('date').rank(method='min')
    count=df.groupby('date', as_index=False).count()[['date', 'jjdm']]
    count.rename(columns={'jjdm':'count'},inplace=True)
    df=pd.merge(df,count,how='left',on='date')
    df['hsl_rank']=df['hsl_rank']/df['count']
    df.to_sql('factor_hsl',con=localdb,index=False,if_exists='append')

def save_fund_return_factor2db():

    jjdm_list=util.get_mutual_stock_funds('20211231')

    #get year end date

    sql_script = "SELECT JYRQ, SFJJ, SFYM,SFZM FROM funddb.JYRL WHERE JYRQ >= {0} and JYRQ <= {1} and SFJJ=0 ".format(
        '20141231', datetime.datetime.today().strftime('%Y%m%d'))
    trade_calander = hbdb.db2df(sql_script, db='readonly').drop('ROW_ID',axis=1).sort_values('JYRQ')
    month_end_list=trade_calander[trade_calander['SFYM']=='1']['JYRQ'].tolist()
    #
    # trade_calander['year'] = trade_calander['JYRQ'].astype(str).str[0:4]
    # year_end_list=trade_calander.groupby('year')['JYRQ'].max().tolist()

    #get month end jj fqnav

    jj_nav=get_jjnav_bydate(jjdm_list,month_end_list)
    jj_nav['month'] = jj_nav['jzrq'].astype(str).str[4:6]
    jj_nav['date']=jj_nav['jzrq']
    jj_nav.set_index('jzrq',inplace=True)
    #get jj year return for each month end date


    jj_nav['year_logret']=np.log(jj_nav.groupby('jjdm')['fqdwjz'].pct_change(12)+1)
    jj_nav['2year_logret'] = np.log(jj_nav.groupby('jjdm')['fqdwjz'].pct_change(24) + 1)
    jj_nav['3year_logret'] = np.log(jj_nav.groupby('jjdm')['fqdwjz'].pct_change(36) + 1)

    jj_nav['year_end_flag']=0
    jj_nav['quarter_end_flag'] = 0
    jj_nav.loc[jj_nav['month']=='12','year_end_flag']=1
    jj_nav.loc[jj_nav['month'].isin(['03','06','09','12'])
    , 'quarter_end_flag'] = 1
    jj_nav['date']=jj_nav['date'].astype(str)

    #localdb.execute('delete from factor_year_ret')
    jj_nav[jj_nav.notnull()].rename(columns={'jzrq':'date'})[['jjdm'
        , 'year_logret','2year_logret','3year_logret', 'date','year_end_flag', 'quarter_end_flag']].to_sql('factor_year_ret'
                                                                       ,con=localdb
                                                                       ,index='False'
                                                                       ,if_exists='append')


def save_fund_rank_factor2db():

    #get the quarter end date
    sql= "SELECT JYRQ, SFYM FROM funddb.JYRL WHERE JYRQ >= {0} and JYRQ<={1} and SFJJ=0 and SFYM=1".format(
        '20160101',datetime.datetime.today().strftime('%Y%m%d'))
    calander = hbdb.db2df(sql, db='readonly').drop('ROW_ID',axis=1)
    calander['month']=calander['JYRQ'].astype(str).str[4:6]
    calander=calander.loc[calander['month'].isin(['03','06','09','12'])]['JYRQ'].tolist()
    calander.sort()

    result=[[],[],[]]

    for date in calander:
        fund_pool=util.get_mutual_stock_funds(date,2)
        sql = "select jjdm,tjyf,rqzh,hb1y from st_fund.t_st_gm_yhb where jjdm in ({0}) and rq1y!=99999 {1} {2} " \
            .format(util.list_sql_condition(fund_pool)
                    , "and rqzh>='{}'".format( str(int(date[0:4])-5)+date[4:6]+'01')
                    , "and rqzh<='{}'".format(date))
        montly_ret = hbdb.db2df(sql, db='funduser')

        #get quarter,half_yearly,yearly return
        montly_ret['hb1q']=montly_ret.groupby('jjdm')['hb1y'].rolling(3).apply(get_quarter_ret).values
        montly_ret['hb1hy']=montly_ret.groupby('jjdm')['hb1y'].rolling(6).apply(get_quarter_ret).values
        montly_ret['hb1y']=montly_ret.groupby('jjdm')['hb1y'].rolling(12).apply(get_quarter_ret).values
        montly_ret['month']=montly_ret['rqzh'].astype(str).str[4:6]
        first_month=montly_ret['tjyf'].min()

        montly_ret=montly_ret.loc[(montly_ret['month'].isin(['03','06','09','12']))&(montly_ret['tjyf']!=first_month)]
        montly_ret['ret_q_rank_per_quarter']=montly_ret.groupby('tjyf').rank(method='min')['hb1q']
        montly_ret['ret_hy_rank_per_quarter']=montly_ret.groupby('tjyf').rank(method='min')['hb1hy']
        montly_ret['ret_y_rank_per_quarter']=montly_ret.groupby('tjyf').rank(method='min')['hb1y']

        montly_ret=pd.merge(montly_ret
                            ,montly_ret.groupby('tjyf').count()['jjdm'].to_frame('count')
                            ,how='left',on='tjyf')

        i=0
        for col in ['ret_q_rank_per_quarter','ret_hy_rank_per_quarter','ret_y_rank_per_quarter']:
            montly_ret[col]=montly_ret[col]/montly_ret['count']
            tempdf=montly_ret.groupby('jjdm')[col].describe()
            tempdf['date']=date
            result[i].append(tempdf.reset_index())
            i+=1

    name_list = ['quarter', 'half_year', 'year']

    for i in range(3):

        sql="delete from factor_{}_rank_quantile".format(name_list[i])
        localdb.execute(sql,con=localdb)
        pd.concat(result[i],axis=0).to_sql('factor_{}_rank_quantile'.format(name_list[i])
                                           ,con=localdb,index=False,if_exists='append')





if __name__ == '__main__':


    #save_fund_rank_factor2db()

    save_fund_return_factor2db()

    # df=read_factors('factor_hsl')
    # df=df.groupby('jjdm').min()
    # df=df[df['hsl'] >= 350]
    #
    # jjdm_list=df.index.tolist()
    # jjdm_con=util.list_sql_condition(jjdm_list)
    #
    # sql="""select jjdm,hb1n,rqzh from st_fund.t_st_gm_nhb
    # where hb1n!=99999 and jjdm in ({0}) and tjnf in ('2015','2016','2017','2018','2019','2020','2021')"""\
    #     .format(jjdm_con)
    # ret=hbdb.db2df(sql,db='funduser')
    # ret=ret.groupby('jjdm')['hb1n'].apply(bhar)
    # ret.to_csv('hlsret.csv',encoding='gbk')
    # print('')


"""
货币类指标
"""
import pandas as pd
from datetime import datetime
from hbshare.quant.Kevin.asset_allocation.macro_index.util import create_table, delete_duplicate_records, WriteToDB
from WindPy import w

w.start()


class Currency:
    def __init__(self, start_date, end_date, is_increment=1):
        self.start_date = start_date
        self.end_date = end_date
        self.is_increment = is_increment
        self.table_name = 'mac_curr'

    def get_currency_data(self):
        """
        货币类数据：M1同比、M2同比、金融机构：短期贷款余额、金融机构：中长期贷款余额、社会融资规模存量同比、社会融资规模存量、
                 7日逆回购利率、贷款市场报价利率(LPR):1年、贷款市场报价利率(LPR):5年、
                 中长期贷款利率:3至5年(含)、人民币存款准备金率:中小型存款类金融机构(月)、人民币存款准备金率:大型存款类金融机构(月)、
                 银行间质押式回购加权利率:7天、
                 基础货币余额、货币乘数
        """
        index_list = ['M0001383', 'M0001385', 'M0043417', 'M0043418', 'M5525763', 'M5525755',
                      'M0041371', 'M0096870', 'M0331299',
                      'M0009815', 'M0043821', 'M0061518',
                      'M0041653',
                      'M0001690', 'M0010131']
        name_dict = {"M0001383": "M1_yoy", "M0001385": "M2_yoy",
                     "M0043417": "short_term_loan_balance", "M0043418": "long_term_loan_balance",
                     "M5525763": "social_finance_yoy", "M5525755": "social_finance_scale",
                     "M0041371": "reverse_repo_7", "M0096870": "LPR_1_year", "M0331299": "LPR_5_year",
                     "M0009815": "long_term_loan_rate",
                     "M0043821": "deposit_reserve_ratio_s", "M0061518": "deposit_reserve_ratio_l",
                     "M0041653": "pledged_repo_7",
                     "M0001690": "currency", "M0010131": "curr_multiplier"}

        res = w.edb(','.join(index_list), self.start_date, self.end_date)
        if res.ErrorCode != 0:
            data = pd.DataFrame()
            print("fetch currency data error: start_date = {}, end_date = {}".format(self.start_date, self.end_date))
        else:
            if len(res.Data) == 1:
                data = pd.DataFrame(res.Data[0], index=res.Codes, columns=res.Times).T
            else:
                data = pd.DataFrame(res.Data, index=res.Codes, columns=res.Times).T
            data.index.name = 'trade_date'
            data.reset_index(inplace=True)
            data['trade_date'] = data['trade_date'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
            data.rename(columns=name_dict, inplace=True)

        data['currency'] = data['currency'].round(2)
        data['curr_multiplier'] = data['curr_multiplier'].round(2)

        return data

    def get_construct_result(self):
        if self.is_increment == 1:
            data = self.get_currency_data()
            sql_script = "delete from {} where trade_date in ({})".format(
                self.table_name, ','.join(data['trade_date'].tolist()))
            # delete first
            delete_duplicate_records(sql_script)
            # add new records
            WriteToDB().write_to_db(data, self.table_name)
        else:
            sql_script = """
                create table mac_curr(
                id int auto_increment primary key,
                trade_date date not null unique,
                M1_yoy decimal(4, 2),
                M2_yoy decimal(4, 2),
                short_term_loan_balance decimal(10, 2),
                long_term_loan_balance decimal(10, 2),
                social_finance_yoy decimal(4, 2),
                social_finance_scale decimal(6, 2),
                reverse_repo_7 decimal(4, 2),
                LPR_1_year decimal(4, 2),
                LPR_5_year decimal(4, 2),
                long_term_loan_rate decimal(4, 2),
                deposit_reserve_ratio_s decimal(4, 2),
                deposit_reserve_ratio_l decimal(4, 2),
                pledged_repo_7 decimal(6, 4),
                currency decimal(9, 2),
                curr_multiplier decimal(4, 2)) 
            """
            create_table(self.table_name, sql_script)
            data = self.get_currency_data()
            WriteToDB().write_to_db(data, self.table_name)


class CurrencyShibor:
    def __init__(self, start_date, end_date, is_increment=1):
        self.start_date = start_date
        self.end_date = end_date
        self.is_increment = is_increment
        self.table_name = 'mac_curr_shibor'

    def get_currency_data(self):
        """
        货币类数据-SHIBOR: 隔夜、一周、两周、一个月、3个月、6个月、9个月、1年
        """
        index_list = ['M0017138', 'M0017139', 'M0017140', 'M0017141', 'M0017142', 'M0017143', 'M0017144', 'M0017145']
        name_dict = {"M0017138": "SHIBOR_ON", "M0017139": "SHIBOR_1W",
                     "M0017140": "SHIBOR_2W", "M0017141": "SHIBOR_1M",
                     "M0017142": "SHIBOR_3M", "M0017143": "SHIBOR_6M",
                     "M0017144": "SHIBOR_9M", "M0017145": "SHIBOR_1Y"}

        res = w.edb(','.join(index_list), self.start_date, self.end_date)
        if res.ErrorCode != 0:
            data = pd.DataFrame()
            print("fetch currency data error: start_date = {}, end_date = {}".format(self.start_date, self.end_date))
        else:
            if len(res.Data) == 1:
                data = pd.DataFrame(res.Data[0], index=res.Codes, columns=res.Times).T
            else:
                data = pd.DataFrame(res.Data, index=res.Codes, columns=res.Times).T
            data.index.name = 'trade_date'
            data.reset_index(inplace=True)
            data['trade_date'] = data['trade_date'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
            data.rename(columns=name_dict, inplace=True)

        return data

    def get_construct_result(self):
        if self.is_increment == 1:
            data = self.get_currency_data()
            sql_script = "delete from {} where trade_date in ({})".format(
                self.table_name, ','.join(data['trade_date'].tolist()))
            # delete first
            delete_duplicate_records(sql_script)
            # add new records
            WriteToDB().write_to_db(data, self.table_name)
        else:
            sql_script = """
                create table mac_curr_shibor(
                id int auto_increment primary key,
                trade_date date not null unique,
                SHIBOR_ON decimal(6, 4),
                SHIBOR_1W decimal(6, 4),
                SHIBOR_2W decimal(6, 4),
                SHIBOR_1M decimal(6, 4),
                SHIBOR_3M decimal(6, 4),
                SHIBOR_6M decimal(6, 4),
                SHIBOR_9M decimal(6, 4),
                SHIBOR_1Y decimal(6, 4)) 
            """
            create_table(self.table_name, sql_script)
            data = self.get_currency_data()
            WriteToDB().write_to_db(data, self.table_name)


if __name__ == '__main__':
    Currency('2005-01-01', '2021-06-10', is_increment=0).get_construct_result()
    # CurrencyShibor('2006-10-08', '2021-05-17', is_increment=1).get_construct_result()


# -*- coding: utf-8 -*-

from hbshare.fe.xwq.analysis.orm.fedb import FEDB
import numpy as np

df1 = FEDB().read_hbs_his_data('hbs_cen_shift_ratio_his_style')
df2 = FEDB().read_hbs_his_data('hbs_cen_shift_ratio_his_industry1')
df3 = FEDB().read_hbs_his_data('hbs_stock_cenlv_his')
df1['jjdm'] = df1['jjdm'].apply(lambda x: str(x).zfill(6))
df2['jjdm'] = df2['jjdm'].apply(lambda x: str(x).zfill(6))
df3['jjdm'] = df3['jjdm'].apply(lambda x: str(x).zfill(6))
df = df1.merge(df2, on=['jjdm', 'jsrq'], how='outer').merge(df3, on=['jjdm', 'jsrq'], how='outer')
df = df[['jjdm', 'jsrq', 'shift_ratio_size', 'c_level_size', 'shift_ratio_style', 'c_level_style', 'shift_ratio_theme', 'c_level_theme', 'shift_ratio', 'c_level', 'cenlv']]
df.columns = ['jjdm', 'jsrq', 'shift_ratio_size', 'c_level_size', 'shift_ratio_style', 'c_level_style', 'shift_ratio_theme', 'c_level_theme', 'shift_ratio_ind', 'c_level_ind', 'c_level_stock']
df_count = df[['jsrq', 'jjdm']].groupby('jsrq').count().reset_index().rename(columns={'jjdm': 'count'})
df = df.merge(df_count, on=['jsrq'], how='left')
for col in list(df.columns)[2:-1]:
    df[col] = df[['jsrq', col]].groupby('jsrq').rank(method='min')
    df[col] = df[col] / df['count']
    df[col] = df[col].replace(np.nan, None)
df = df.drop('count', axis=1)
FEDB().insert_hbs_rank_his_df(df)


# -*- coding: utf-8 -*-

from pdfminer.pdfparser import PDFParser
from pdfminer.pdfdocument import PDFDocument
from pdfminer.pdfpage import PDFPage, PDFTextExtractionNotAllowed
from pdfminer.pdfinterp import PDFResourceManager, PDFPageInterpreter
from pdfminer.converter import PDFPageAggregator
from pdfminer.layout import LAParams
from io import StringIO

# from io import StringIO
# from pdfminer.pdfinterp import PDFResourceManager
# from pdfminer.pdfinterp import process_pdf
# from pdfminer.converter import TextConverter
# from pdfminer.layout import LAParams
# import re

import pdfplumber
import pandas as pd


def performance(content):
    return


def read_from_pdfplumber(file_path, is_cover=False, is_catalog=False, is_content=False, page_min=None, page_max=None):
    """
    读取pdf文件
    """
    with pdfplumber.open(file_path) as file:
        total_pages = len(file.pages)
        content = ''
        if is_cover:
            content += file.pages[0].extract_text()
            return content
        if is_catalog:
            first_catalog = file.pages[1].extract_text()
            catalog_page_min = int(first_catalog.split('\n')[2].split(' ')[-1])
            catalog_page_max = int(first_catalog.split('\n')[3].split(' ')[-1])
            for i in range(catalog_page_min - 1, catalog_page_max - 1):
                content += file.pages[i].extract_text()
        if is_content:
            for i in range(page_min - 1, page_max):
                content += file.pages[i].extract_text()
        return content

def pdf_analysis(data_path, file_name):
    cover = read_from_pdfplumber('{0}{1}'.format(data_path, file_name), is_cover=True)
    catalog = read_from_pdfplumber('{0}{1}'.format(data_path, file_name), is_catalog=True)
    sector_page = pd.DataFrame(catalog.split('\n')).iloc[3:-3].drop_duplicates()
    sector_page = sector_page[sector_page[0].apply(lambda x: True if len(x.split(' ')) == 2 else False)]
    sector_page['SECTOR_NAME'] = sector_page[0].apply(lambda x: x.split(' ')[0])
    sector_page['PAGE_MIN'] = sector_page[0].apply(lambda x: x.split(' ')[1])
    sector_page = sector_page.set_index('SECTOR_NAME')[['PAGE_MIN']]
    sector_page['PAGE_MAX'] = sector_page['PAGE_MIN'].shift(-1)
    sector_page = sector_page.dropna()
    for sector_name in list(sector_page.index):
        page_min = int(sector_page.loc[sector_name]['PAGE_MIN'])
        page_max = int(sector_page.loc[sector_name]['PAGE_MAX'])
        content = read_from_pdfplumber('{0}{1}'.format(data_path, file_name), is_content=True, page_min=page_min, page_max=page_max)
    return

def test_PyPDF2(data_path, file_name):
    from PyPDF2 import PdfFileReader
    with open('{0}{1}'.format(data_path, file_name), 'rb') as f:
        file = PdfFileReader(f)
        info = file.getDocumentInfo()
        pages = file.getNumPages()
        content = ''
        for page in range(pages):
            content += file.getPage(page).extractText()
    return

def test_pdfrw(data_path, file_name):
    from pdfrw import PdfReader
    file = PdfReader('{0}{1}'.format(data_path, file_name))
    keys = file.keys()
    info = file.Info
    root_keys = file.Root.keys()
    pages = len(file.pages)
    content = ''
    for page in range(pages):
        content += file.pages[page].Contents.stream
    return

def test_pikepdf(data_path, file_name):
    import pikepdf
    with pikepdf.open('{0}{1}'.format(data_path, file_name)) as file:
        info = file.docinfo
        pages = len(file.pages)
        content = ''
        for page in range(pages):
            content += file.pages[page]
    return

def test_pdfminer3k(data_path, file_name):
    output = StringIO()
    with open('{0}{1}'.format(data_path, file_name), 'rb') as f:
        praser = PDFParser(f)
        doc = PDFDocument(praser)
        if not doc.is_extractable:
            raise PDFTextExtractionNotAllowed
        pdfrm = PDFResourceManager()
        laparams = LAParams()
        device = PDFPageAggregator(pdfrm, laparams=laparams)
        interpreter = PDFPageInterpreter(pdfrm, device)
        for page in PDFPage.create_pages(doc):
            interpreter.process_page(page)
            layout = device.get_result()
            for x in layout:
                if hasattr(x, "get_text"):
                    content = x.get_text()
                    output.write(content)
    content = output.getvalue()
    output.close()
    return

def test_pdfplumber(data_path, file_name):
    import pdfplumber
    with pdfplumber.open('{0}{1}'.format(data_path, file_name)) as file:
        pages = len(file.pages)
        content = ''
        for page in range(pages):
            content += file.pages[page].extract_text()
    return

def test_PyMuPDF(data_path, file_name):
    import fitz
    file = fitz.open('{0}{1}'.format(data_path, file_name))
    pages = file.page_count
    metadata = file.metadata
    toc = file.get_toc()
    content = ''
    for page in range(pages):
        page_content = file.load_page(page)
        content += page_content.get_text('text')
    return

if __name__ == '__main__':
    data_path = 'D:/Git/hbshare/hbshare/fe/xwq/data/pdf/'
    file_name = '上海衍复投资管理有限公司_自定义报告_衍复臻选中证1000指数增强一号私募证券投资基金（20220201-20220228）.pdf'
    pdf_analysis(data_path, file_name)

    # test_PyPDF2(data_path, file_name)
    # test_pdfrw(data_path, file_name)
    # test_pikepdf(data_path, file_name)
    # test_pdfminer3k(data_path, file_name)
    test_pdfplumber(data_path, file_name)  # pd.DataFrame(file.pages[4].extract_tables()).T
    # test_PyMuPDF(data_path, file_name)



import datetime
import pandas as pd
import numpy as np
from hbshare.fe.XZ import db_engine
from hbshare.fe.XZ import functionality
from hbshare.fe.mutual_analysis import  jj_picturing as jjpic
# from ortools.linear_solver import  pywraplp
from PIL import Image
from docx import Document
from docx.shared import Inches,Pt
from docx.enum.text import WD_PARAGRAPH_ALIGNMENT


util=functionality.Untils()
hbdb=db_engine.HBDB()
localdb=db_engine.PrvFunDB().engine
plot=functionality.Plot(1000,600)



def get_core_pool():

    sql="select jjdm,jjjc from st_fund.t_st_gm_jjxx where hxbz='1' and cpfl='2' "
    pool=hbdb.db2df(sql,db='funduser')

    return  pool

def pool_his_fromexcel2db():

    df_in=pd.read_excel(r"C:\Users\xuhuai.zhe\Downloads\公募股多核心池20220429.xlsx",
                     sheet_name='核心池-主动股多').sort_values('调入时间').drop(['备注','研究员'],axis=1)
    df_out=pd.read_excel(r"C:\Users\xuhuai.zhe\Downloads\公募股多核心池20220429.xlsx",
                     sheet_name='出池').sort_values('调出时间').drop('备注',axis=1)
    df_in['调入时间']=df_in['调入时间'].astype(str).str[0:7].str.replace('-', '')
    df_in['调出时间']=''
    df_out['调出时间'] = df_out['调出时间'].astype(str).str[0:7].str.replace('-', '')
    df_out['调入时间'] = df_out['调入时间'].astype(str).str[0:7].str.replace('-', '')
    df=pd.concat([df_in,df_out],axis=0).sort_values('调入时间')
    df.reset_index(drop=True,inplace=True)
    df['基金代码']=df['基金代码'].astype(str).str[0:6]

    poo_his=pd.DataFrame()
    for year in ['2020','2021','2022']:
        for month in ['03','06','09','12']:
            tempdf=df[(df['调入时间']<=year+month)&((df['调出时间']=='')|(df['调出时间']>year+month))][['基金代码','基金名称','基金经理']]
            tempdf['asofdate']=year+month
            poo_his=pd.concat([poo_his,tempdf],axis=0)

    poo_his.reset_index(drop=True,inplace=True)
    poo_his=poo_his[poo_his['asofdate']<=str(datetime.datetime.now())[0:7].replace('-','')]
    sql='delete from core_pool_history'
    localdb.execute(sql)
    poo_his.to_sql('core_pool_history',con=localdb,if_exists='append',index=False)

def get_core_poolfromlocaldb(asofdate=None):

    if(asofdate is not None):
        sql="select * from core_pool_history where asofdate='{}'".format(asofdate)
    else:
        sql="select * from core_pool_history"
    pool=pd.read_sql(sql,con=localdb).rename(columns={'基金代码':'jjdm',
                                                      '基金名称':'jjjc',
                                                      '基金经理':'manager'})

    return pool

def get_jj_picture(jjdm_con=None,if_inverse=False):

    if(if_inverse):
        inornot='not in '
    else:
        inornot='in'

    value_df_list = [pd.DataFrame(), pd.DataFrame(), pd.DataFrame(), pd.DataFrame(),
                     pd.DataFrame(), pd.DataFrame(), pd.DataFrame(), pd.DataFrame()]

    industry_df_list = [pd.DataFrame(), pd.DataFrame(), pd.DataFrame(), pd.DataFrame(), pd.DataFrame()]


    stock_df_list = [pd.DataFrame(), pd.DataFrame()]


    if(jjdm_con is not None):
        value_p, value_p_hbs, value_sp, value_sp_hbs, size_p, size_p_hbs, size_sp, size_sp_hbs, \
        industry_p, industry_sp, theme_p, theme_sp, industry_detail_df_list, stock_p, stock_tp=\
            jjpic.get_pic_from_localdb("jjdm {1} ({0})".format(jjdm_con,inornot), if_percentage=False)
    else:
        value_p, value_p_hbs, value_sp, value_sp_hbs, size_p, size_p_hbs, size_sp, size_sp_hbs, \
        industry_p, industry_sp, theme_p, theme_sp, industry_detail_df_list, stock_p, stock_tp=\
            jjpic.get_pic_from_localdb("1=1", if_percentage=False)

    value_df_list[0]=value_p
    value_df_list[1] = value_p_hbs
    value_df_list[2] = value_sp
    value_df_list[3] = value_sp_hbs
    value_df_list[4] = size_p
    value_df_list[5] = size_p_hbs
    value_df_list[6] = size_sp
    value_df_list[7] = size_sp_hbs

    industry_df_list[0]=industry_p
    industry_df_list[1] = industry_sp
    industry_df_list[2] = theme_p
    industry_df_list[3] = theme_sp
    for j in range(len(industry_detail_df_list)):
        # industry_detail_df_list[j]['jjdm'] = jjdm
        industry_detail_df_list[j]['industry_level'] = j + 1
        industry_df_list[4] = pd.concat([industry_df_list[4], industry_detail_df_list[j]], axis=0)

    stock_df_list[0]=stock_p
    stock_df_list[1] = stock_tp

    return value_df_list,industry_df_list,stock_df_list

def features_calculation(value_df_list, industry_df_list, stock_df_list):

    industry_style=industry_df_list[0].groupby('一级行业类型').count()['jjdm'].to_frame('num')
    industry_style=industry_style/len(industry_df_list[0])*100
    industry_style_dis = (industry_df_list[4].groupby(['industry_level', '行业名称']).mean() * 100).reset_index()[
        ['industry_level', '行业名称', '占持仓比例(时序均值)', '占持仓比例(时序均值)排名']]
    #take industry with weight more than 5%
    industry_style_dis=industry_style_dis[industry_style_dis['占持仓比例(时序均值)'] >= (5/industry_style_dis['industry_level'])]
    class_list=[]
    for i in range(3):
        class_list.append(industry_style_dis[industry_style_dis['industry_level'] == (i + 1)] \
            .sort_values('占持仓比例(时序均值)', ascending=False).set_index('行业名称').drop('industry_level', axis=1))

    style_type_dis = value_df_list[1].groupby('风格类型').count()['jjdm'].to_frame('风格类型') \
                     / len(value_df_list[1]) * 100
    style_incline_dis=value_df_list[1][value_df_list[1]['风格类型']=='专注'][['风格类型','风格偏好']]
    style_incline_dis=style_incline_dis.groupby('风格偏好').count()/len(style_incline_dis)*100
    style_weight_dis=value_df_list[1][['成长绝对暴露(持仓)',
       '价值绝对暴露(持仓)','成长暴露排名(持仓)', '价值暴露排名(持仓)']].mean(axis=0).to_frame('成长价值权重分布')*100

    style_type_dis2 = value_df_list[5].groupby('规模风格类型').count()['jjdm'].to_frame('规模风格类型') \
                      / len(value_df_list[5]) * 100
    style_incline_dis2=value_df_list[5][value_df_list[5]['规模风格类型']=='专注'][['规模风格类型','规模偏好']]
    style_incline_dis2=style_incline_dis2.groupby('规模偏好').count()/len(style_incline_dis2)*100
    style_weight_dis2=value_df_list[5][['大盘绝对暴露(持仓)',
       '中盘绝对暴露(持仓)', '小盘绝对暴露(持仓)','大盘暴露排名(持仓)',
       '中盘暴露排名(持仓)', '小盘暴露排名(持仓)']].mean(axis=0).to_frame('成长价值权重分布')*100

    stock_style_a=stock_df_list[0].groupby('个股风格A').count()['jjdm'].to_frame('个股风格类型A')\
                  /len(stock_df_list[0])*100
    stock_style_b = stock_df_list[0].groupby('个股风格B').count()['jjdm'].to_frame('个股风格类型B')\
                    /len(stock_df_list[0])*100
    stock_left = stock_df_list[1].groupby('左侧标签').count()['jjdm'].to_frame('左侧类型分布')\
                 / len(stock_df_list[1]) * 100
    stock_df_list[1].loc[stock_df_list[1]['新股次新股偏好'] == '', '新股次新股偏好'] = '无'
    stock_new=stock_df_list[1].groupby('新股次新股偏好').count()['jjdm'].to_frame('新股偏好分布')\
              / len(stock_df_list[1]) * 100
    stock_fin2=(stock_df_list[1][['平均持有时间(出持仓前)_rank', '左侧概率(出重仓前,半年线)_rank','新股概率(出持仓前)_rank'
        ,'出重仓前平均收益率_rank',
       '出全仓前平均收益率_rank',]]).mean(axis=0)
    stock_fin=(stock_df_list[0][['PE_rank', 'PB_rank',
       'ROE_rank', '股息率_rank','平均仓位']]).mean(axis=0)
    stock_fin=pd.concat([stock_fin,stock_fin2],axis=0).to_frame('个股持仓特征')
    stock_fin.index = [x.replace('rank', '排名') for x in stock_fin.index]
    stock_fin=stock_fin*100


    return industry_style,class_list,style_type_dis,style_incline_dis,\
           style_weight_dis,style_type_dis2,style_incline_dis2,style_weight_dis2,\
           stock_style_a,stock_style_b,stock_left,stock_new,stock_fin

def pool_industry(industry_style,class_list,save_local_file=False):

    if(len(industry_style.columns)>1):
        plot.plotly_jjpic_bar(industry_style, '行业风格分布',save_local_file=save_local_file)
    else:
        plot.plotly_pie(industry_style, '行业风格分布',save_local_file=save_local_file)

    for i in range(3):

        class_list[i].columns=[x.replace('占持仓比例(时序均值)','') for x in class_list[i].columns]

        plot.plotly_jjpic_bar(class_list[i][class_list[i].columns[0:int(len(class_list[i].columns)/2)]].fillna(0),"{}级行业持仓分布_占持仓比例".format(i+1),save_local_file=save_local_file)
        plot.plotly_jjpic_bar(class_list[i][class_list[i].columns[int(len(class_list[i].columns)/2):]].fillna(0), "{}级行业持仓分布_占持仓比例排名".format(i + 1),save_local_file=save_local_file)

def pool_style(style_type_dis,style_incline_dis,
               style_weight_dis,style_type_dis2,style_incline_dis2,style_weight_dis2,save_local_file=False):

    if(len(style_type_dis.columns)>1):
        plot.plotly_jjpic_bar(style_type_dis, '风格类型分布',save_local_file=save_local_file)
        plot.plotly_jjpic_bar(style_incline_dis, '风格专注型分布',save_local_file=save_local_file)
        plot.plotly_jjpic_bar(style_weight_dis.iloc[0:2], '成长价值持仓占比',save_local_file=save_local_file)
        plot.plotly_jjpic_bar(style_weight_dis.iloc[2:], '成长价值持仓占比排名',save_local_file=save_local_file)

        plot.plotly_jjpic_bar(style_type_dis2, '规模风格类型分布',save_local_file=save_local_file)
        plot.plotly_jjpic_bar(style_incline_dis2, '专注型风格分布',save_local_file=save_local_file)
        plot.plotly_jjpic_bar(style_weight_dis2.iloc[0:3], '大中小盘持仓占比',save_local_file=save_local_file)
        plot.plotly_jjpic_bar(style_weight_dis2.iloc[3:], '大中小盘持仓占比排名',save_local_file=save_local_file)
    else:
        plot.plotly_pie(style_type_dis,'风格类型分布',save_local_file=save_local_file)
        plot.plotly_pie(style_incline_dis,'风格专注型分布',save_local_file=save_local_file)
        plot.plotly_pie(style_weight_dis.iloc[0:2],'成长价值持仓占比',save_local_file=save_local_file)
        plot.plotly_pie(style_weight_dis.iloc[2:],'成长价值持仓占比排名',save_local_file=save_local_file)


        plot.plotly_pie(style_type_dis2,'规模风格类型分布',save_local_file=save_local_file)
        plot.plotly_pie(style_incline_dis2,'专注型风格分布',save_local_file=save_local_file)
        plot.plotly_pie(style_weight_dis2.iloc[0:3],'大中小盘持仓占比',save_local_file=save_local_file)
        plot.plotly_pie(style_weight_dis2.iloc[3:],'大中小盘持仓占比排名',save_local_file=save_local_file)

def pool_other(stock_style_a,stock_style_b,stock_left,stock_new,stock_fin,save_local_file=False):

    if(len(stock_style_a.columns)>1):
        plot.plotly_jjpic_bar(stock_style_a,'个股风格类型分布A',save_local_file=save_local_file)
        plot.plotly_jjpic_bar(stock_style_b,'个股风格类型分布B',save_local_file=save_local_file)
        plot.plotly_jjpic_bar(stock_left ,'左侧类型分布',save_local_file=save_local_file)
        plot.plotly_jjpic_bar(stock_new,'新股偏好分布',save_local_file=save_local_file)
    else:
        plot.plotly_pie(stock_style_a,'个股风格类型分布A',save_local_file=save_local_file)
        plot.plotly_pie(stock_style_b,'个股风格类型分布B',save_local_file=save_local_file)
        plot.plotly_pie(stock_left ,'左侧类型分布',save_local_file=save_local_file)
        plot.plotly_pie(stock_new,'新股偏好分布',save_local_file=save_local_file)
    plot.plotly_jjpic_bar(stock_fin,'个股持仓特征',save_local_file=save_local_file)

def pool_picturing_engine(industry_style, class_list, style_type_dis, style_incline_dis, \
    style_weight_dis, style_type_dis2, style_incline_dis2, style_weight_dis2, \
    stock_style_a, stock_style_b, stock_left, stock_new, stock_fin,save_local_file=False):

    pool_industry(industry_style,class_list,save_local_file)
    pool_style(style_type_dis,style_incline_dis,
               style_weight_dis,style_type_dis2,style_incline_dis2,style_weight_dis2,save_local_file)
    pool_other(stock_style_a,stock_style_b,stock_left,stock_new,stock_fin,save_local_file)


def get_potient_pool(jjdm_con):

    value_df_list,industry_df_list,stock_df_list=get_jj_picture(jjdm_con,if_inverse=True)

    return value_df_list,industry_df_list,stock_df_list

# def optimizer_allocation(value_df_list,industry_df_list, stock_df_list,pool,industry_style, class_list, style_type_dis, style_incline_dis, \
#     style_weight_dis, style_type_dis2, style_incline_dis2, style_weight_dis2, \
#     stock_style_a, stock_style_b, stock_left, stock_new, stock_fin):
#
#     turnover_num=20
#     style_order = ['专注', '博弈', '轮动', '配置']
#
#     jjdm_con=util.list_sql_condition(pool['jjdm'].unique().tolist())
#     value_df_list_pot, industry_df_list_pot, stock_df_list_pot=get_potient_pool(jjdm_con)
#
#     potient_jj_list=industry_df_list_pot[0]['jjdm'].tolist()
#
#
#     sql="select jjdm,zxppm1nyj from st_fund.t_st_gm_nxpblpm where jjdm not in ({0}) and tjnf>={1} ".format(jjdm_con,2022-1)
#     sharpr=-1*hbdb.db2df(sql,db='funduser').set_index('jjdm').loc[potient_jj_list]['zxppm1nyj'].astype(float).values
#
#     solver=pywraplp.Solver.CreateSolver('SCIP')
#     x=[]
#     for i in potient_jj_list:
#         x.append(solver.IntVar(0,1,"x_{}".format(i)))
#
#
#     input_industry_style_cons_ub=np.array([0.3,0.3,0.3,0.3])
#     input_industry_style_cons_lb = np.array([0, 0, 0, 0])
#     input_industry_style_cons_ub=input_industry_style_cons_ub*(turnover_num+len(pool))\
#                                  -industry_style.loc[style_order]['num'].values/100*len(pool)
#     input_industry_style_cons_lb=input_industry_style_cons_lb*(turnover_num+len(pool))\
#                                  -industry_style.loc[style_order]['num'].values/100*len(pool)
#
#
#     cons_dict=dict()
#     #set constrains bound
#     cons_dict['turnover_num']=solver.Constraint(turnover_num, turnover_num)
#     for m in range(4):
#         cons_dict['industry_style_{}'.format(style_order[m])] = solver.Constraint(input_industry_style_cons_lb[m], input_industry_style_cons_ub[m])
#     industry_style_cofe=pd.get_dummies(industry_df_list_pot[0].set_index('jjdm')
#                                        .loc[potient_jj_list]['一级行业类型'])[style_order].astype(float).values
#
#     for i in range(len(x)):
#         cons_dict['turnover_num'].SetCoefficient(x[i], 1)
#         for m in range(4):
#             cons_dict['industry_style_{}'.format(style_order[m])].SetCoefficient(x[i], industry_style_cofe[i,m])
#
#     objective = solver.Objective()
#     for i in range(len(x)):
#         objective.SetCoefficient(x[i], sharpr[i])
#     objective.SetMaximization()
#
#     result_status = solver.Solve()
#     print(result_status == pywraplp.Solver.OPTIMAL)
#     print(solver.Objective().Value())
#
#     adding_list=[]
#     for i in range(len(x)):
#         if(x[i].solution_value()==1):
#             adding_list.append(str(x[i]).split('_')[-1])
#
#     new_jjdm_list=pool['jjdm'].unique().tolist()+adding_list
#
#     return  new_jjdm_list

def pool_picturing_opt():

    pool = get_core_pool()
    pool=get_core_poolfromlocaldb('202203')
    #picture the original pool
    jjdm_con = util.list_sql_condition(pool['jjdm'].unique().tolist())
    value_df_list, industry_df_list, stock_df_list = get_jj_picture(jjdm_con)
    industry_style, class_list, style_type_dis, style_incline_dis, \
    style_weight_dis, style_type_dis2, style_incline_dis2, style_weight_dis2, \
    stock_style_a, stock_style_b, stock_left, stock_new, stock_fin\
        =features_calculation(value_df_list, industry_df_list, stock_df_list)

    pool_picturing_engine(industry_style, class_list, style_type_dis, style_incline_dis, \
    style_weight_dis, style_type_dis2, style_incline_dis2, style_weight_dis2, \
    stock_style_a, stock_style_b, stock_left, stock_new, stock_fin)

    # new_jjdm_list=optimizer_allocation(value_df_list, industry_df_list, stock_df_list,pool,industry_style, class_list, style_type_dis, style_incline_dis, \
    # style_weight_dis, style_type_dis2, style_incline_dis2, style_weight_dis2, \
    # stock_style_a, stock_style_b, stock_left, stock_new, stock_fin)

    #pic the new pool
    #new_jjdm_con=util.list_sql_condition(new_jjdm_list)
    new_jjdm_con="'001705','001410','001856','450009','166006'," \
                 "'001975','003095','005241','161606','005827','002708'," \
                 "'005968','005267','519133','002340','000577','163415'," \
                 "'000729','000756','000991','001208','001487','001532'," \
                 "'001718','001877','002258','002871','002943','004350'," \
                 "'004784','005161','005457','005669','005825','006257','007130','519702'"
    value_df_list, industry_df_list, stock_df_list = get_jj_picture(new_jjdm_con)
    industry_style_new, class_list_new, style_type_dis_new, style_incline_dis_new, \
    style_weight_dis_new, style_type_dis2_new, style_incline_dis2_new, style_weight_dis2_new, \
    stock_style_a_new, stock_style_b_new, stock_left_new, stock_new_new, stock_fin_new\
        =features_calculation(value_df_list, industry_df_list, stock_df_list)

    industry_style_new=pd.merge(industry_style,industry_style_new,how='outer',left_index=True,right_index=True)
    industry_style_new.columns=['旧池','新池']

    for i in range(3):
        class_list_new[i]=pd.merge(class_list[i],class_list_new[i],how='outer',
                                   left_index=True,right_index=True)
        class_list_new[i].columns=[x.replace('x','旧') for x in class_list_new[i].columns]
        class_list_new[i].columns = [x.replace('y', '新') for x in class_list_new[i].columns]
        class_list_new[i]=class_list_new[i][class_list_new[i].columns.sort_values()]


    style_type_dis_new=pd.merge(style_type_dis,style_type_dis_new,how='outer',
                                left_index=True,right_index=True).fillna(0)
    style_type_dis_new.columns=['旧池','新池']

    style_incline_dis_new=pd.merge(style_incline_dis,style_incline_dis_new,how='outer',
                                   left_index=True,right_index=True).fillna(0)
    style_incline_dis_new.columns=['旧池','新池']

    style_weight_dis_new=pd.merge(style_weight_dis,style_weight_dis_new,how='outer',
                                  left_index=True,right_index=True).fillna(0)
    style_weight_dis_new.columns=['旧池','新池']

    style_type_dis2_new=pd.merge(style_type_dis2,style_type_dis2_new,how='outer',
                                 left_index=True,right_index=True).fillna(0)
    style_type_dis2_new.columns=['旧池','新池']

    style_incline_dis2_new=pd.merge(style_incline_dis2,style_incline_dis2_new,
                                    how='outer',left_index=True,right_index=True).fillna(0)
    style_incline_dis2_new.columns=['旧池','新池']

    style_weight_dis2_new=pd.merge(style_weight_dis2,style_weight_dis2_new,
                                   how='outer',left_index=True,right_index=True).fillna(0)
    style_weight_dis2_new.columns=['旧池','新池']

    stock_style_a_new=pd.merge(stock_style_a,stock_style_a_new,
                               how='outer',left_index=True,right_index=True).fillna(0)
    stock_style_a_new.columns=['旧池','新池']

    stock_style_b_new=pd.merge(stock_style_b,stock_style_b_new,how='outer'
                               ,left_index=True,right_index=True).fillna(0)
    stock_style_b_new.columns=['旧池','新池']

    stock_left_new=pd.merge(stock_left,stock_left_new,how='outer'
                            ,left_index=True,right_index=True).fillna(0)
    stock_left_new.columns=['旧池','新池']

    stock_new_new=pd.merge(stock_new,stock_new_new,how='outer',
                           left_index=True,right_index=True).fillna(0)
    stock_new_new.columns=['旧池','新池']

    stock_fin_new=pd.merge(stock_fin,stock_fin_new,how='outer',
                           left_index=True,right_index=True).fillna(0)
    stock_fin_new.columns=['旧池','新池']


    pool_picturing_engine(industry_style_new, class_list_new, style_type_dis_new, style_incline_dis_new, \
    style_weight_dis_new, style_type_dis2_new, style_incline_dis2_new, style_weight_dis2_new, \
    stock_style_a_new, stock_style_b_new, stock_left_new, stock_new_new, stock_fin_new)

def pool_picturing(pool=None,returndata=False,save_local_file=False):

    if(pool is None):
        pool=get_core_poolfromlocaldb('202203')

    #picture the original pool
    jjdm_con = util.list_sql_condition(pool['jjdm'].unique().tolist())
    value_df_list, industry_df_list, stock_df_list = get_jj_picture(jjdm_con)
    industry_style, class_list, style_type_dis, style_incline_dis, \
    style_weight_dis, style_type_dis2, style_incline_dis2, style_weight_dis2, \
    stock_style_a, stock_style_b, stock_left, stock_new, stock_fin\
        =features_calculation(value_df_list, industry_df_list, stock_df_list)

    plot.plotly_table(pool, 1000, '池列表', save_local_file=True)
    pool_picturing_engine(industry_style, class_list, style_type_dis, style_incline_dis, \
    style_weight_dis, style_type_dis2, style_incline_dis2, style_weight_dis2, \
    stock_style_a, stock_style_b, stock_left, stock_new, stock_fin,save_local_file)

    if(returndata):
        return industry_style, class_list, style_type_dis, style_incline_dis, \
        style_weight_dis, style_type_dis2, style_incline_dis2, style_weight_dis2, \
        stock_style_a, stock_style_b, stock_left, stock_new, stock_fin

def pool_comparision(date1,date2):

    pool=get_core_poolfromlocaldb(date1)

    #picture the original pool
    jjdm_con = util.list_sql_condition(pool['jjdm'].unique().tolist())
    value_df_list, industry_df_list, stock_df_list = get_jj_picture(jjdm_con)
    industry_style, class_list, style_type_dis, style_incline_dis, \
    style_weight_dis, style_type_dis2, style_incline_dis2, style_weight_dis2, \
    stock_style_a, stock_style_b, stock_left, stock_new, stock_fin\
        =features_calculation(value_df_list, industry_df_list, stock_df_list)

    # picture the new pool
    new_pool = get_core_poolfromlocaldb(date2)
    new_jjdm_con=util.list_sql_condition(new_pool['jjdm'].unique().tolist())
    value_df_list, industry_df_list, stock_df_list = get_jj_picture(new_jjdm_con)
    industry_style_new, class_list_new, style_type_dis_new, style_incline_dis_new, \
    style_weight_dis_new, style_type_dis2_new, style_incline_dis2_new, style_weight_dis2_new, \
    stock_style_a_new, stock_style_b_new, stock_left_new, stock_new_new, stock_fin_new\
        =features_calculation(value_df_list, industry_df_list, stock_df_list)

    #get the return for in and out jj for the next 1 quarter and year
    in_jjdm=list(set(new_pool['jjdm'].unique().tolist()).difference(set(pool['jjdm'].unique().tolist())))
    out_jjdm=list(set(pool['jjdm'].unique().tolist()).difference(set(new_pool['jjdm'].unique().tolist())))
    start_date=util._shift_date(date2+'31')
    in_ret=pd.DataFrame(data=in_jjdm,columns=['jjdm'])
    out_ret = pd.DataFrame(data=out_jjdm, columns=['jjdm'])

    end_date_y=util._shift_date(str(int(date2[0:4])+1)+date2[4:]+'30')
    if(date2[4:]!='12'):
        end_date_q=util._shift_date(date2[0:4]+("0"+str(int(date2[4:])+3))[-2:]+'30')
    else:
        end_date_q=util._shift_date(str(int(date2[0:4])+1)+'0331')

    if(end_date_y>=datetime.datetime.today().strftime('%Y%m%d')[0:6]):
        end_date_y=util._shift_date(str(hbdb.db2df("select max(jzrq) as jzrq from st_fund.t_st_gm_rhb", db='funduser')['jzrq'][0]))
    sql = "select jjdm,jzrq,fqdwjz from st_fund.t_st_gm_rhb where (jzrq='{0}' or  jzrq='{1}') and jjdm in({2})" \
        .format(end_date_y, start_date, util.list_sql_condition(in_jjdm+out_jjdm))
    gap_return_y = hbdb.db2df(sql, db='funduser')
    gap_return_y=pd.merge(gap_return_y[gap_return_y['jzrq']==int(start_date)]
                          ,gap_return_y[gap_return_y['jzrq']==int(end_date_y)],on='jjdm')
    gap_return_y['ret']=gap_return_y['fqdwjz_y']/gap_return_y['fqdwjz_x']-1


    if(end_date_q>=datetime.datetime.today().strftime('%Y%m%d')[0:6]):
        end_date_q = util._shift_date(str(hbdb.db2df("select max(jzrq) as jzrq from st_fund.t_st_gm_rhb", db='funduser')['jzrq'][0]))
    sql = "select jjdm,jzrq,fqdwjz from st_fund.t_st_gm_rhb where (jzrq='{0}' or  jzrq='{1}') and jjdm in({2})" \
        .format(end_date_q, start_date, util.list_sql_condition(in_jjdm+out_jjdm))
    gap_return_q = hbdb.db2df(sql, db='funduser')
    gap_return_q=pd.merge(gap_return_q[gap_return_q['jzrq']==int(start_date)]
                          ,gap_return_q[gap_return_q['jzrq']==int(end_date_q)],on='jjdm')
    gap_return_q['ret']=gap_return_q['fqdwjz_y']/gap_return_q['fqdwjz_x']-1

    in_ret=pd.merge(in_ret,gap_return_q[['jjdm','ret']],how='left',on='jjdm')\
        .rename(columns={'ret':'ret_q'})
    in_ret=pd.merge(in_ret,gap_return_y[['jjdm','ret']],how='left',on='jjdm')\
        .rename(columns={'ret':'ret_y'})
    in_ret=in_ret.mean()
    out_ret=pd.merge(out_ret,gap_return_q[['jjdm','ret']],how='left',on='jjdm')\
        .rename(columns={'ret':'ret_q'})
    out_ret=pd.merge(out_ret,gap_return_y[['jjdm','ret']],how='left',on='jjdm')\
        .rename(columns={'ret':'ret_y'})
    out_ret = out_ret.mean()

    #join the two pool picture
    industry_style_new=pd.merge(industry_style,industry_style_new,how='outer',left_index=True,right_index=True)
    industry_style_new.columns=['旧池','新池']

    for i in range(3):
        class_list_new[i]=pd.merge(class_list[i],class_list_new[i],how='outer',
                                   left_index=True,right_index=True)
        class_list_new[i].columns=[x.replace('x','旧') for x in class_list_new[i].columns]
        class_list_new[i].columns = [x.replace('y', '新') for x in class_list_new[i].columns]
        class_list_new[i]=class_list_new[i][class_list_new[i].columns.sort_values()]


    style_type_dis_new=pd.merge(style_type_dis,style_type_dis_new,how='outer',
                                left_index=True,right_index=True).fillna(0)
    style_type_dis_new.columns=['旧池','新池']

    style_incline_dis_new=pd.merge(style_incline_dis,style_incline_dis_new,how='outer',
                                   left_index=True,right_index=True).fillna(0)
    style_incline_dis_new.columns=['旧池','新池']

    style_weight_dis_new=pd.merge(style_weight_dis,style_weight_dis_new,how='outer',
                                  left_index=True,right_index=True).fillna(0)
    style_weight_dis_new.columns=['旧池','新池']

    style_type_dis2_new=pd.merge(style_type_dis2,style_type_dis2_new,how='outer',
                                 left_index=True,right_index=True).fillna(0)
    style_type_dis2_new.columns=['旧池','新池']

    style_incline_dis2_new=pd.merge(style_incline_dis2,style_incline_dis2_new,
                                    how='outer',left_index=True,right_index=True).fillna(0)
    style_incline_dis2_new.columns=['旧池','新池']

    style_weight_dis2_new=pd.merge(style_weight_dis2,style_weight_dis2_new,
                                   how='outer',left_index=True,right_index=True).fillna(0)
    style_weight_dis2_new.columns=['旧池','新池']

    stock_style_a_new=pd.merge(stock_style_a,stock_style_a_new,
                               how='outer',left_index=True,right_index=True).fillna(0)
    stock_style_a_new.columns=['旧池','新池']

    stock_style_b_new=pd.merge(stock_style_b,stock_style_b_new,how='outer'
                               ,left_index=True,right_index=True).fillna(0)
    stock_style_b_new.columns=['旧池','新池']

    stock_left_new=pd.merge(stock_left,stock_left_new,how='outer'
                            ,left_index=True,right_index=True).fillna(0)
    stock_left_new.columns=['旧池','新池']

    stock_new_new=pd.merge(stock_new,stock_new_new,how='outer',
                           left_index=True,right_index=True).fillna(0)
    stock_new_new.columns=['旧池','新池']

    stock_fin_new=pd.merge(stock_fin,stock_fin_new,how='outer',
                           left_index=True,right_index=True).fillna(0)
    stock_fin_new.columns=['旧池','新池']



    pool_picturing_engine(industry_style_new, class_list_new, style_type_dis_new, style_incline_dis_new, \
    style_weight_dis_new, style_type_dis2_new, style_incline_dis2_new, style_weight_dis2_new, \
    stock_style_a_new, stock_style_b_new, stock_left_new, stock_new_new, stock_fin_new)


    return  in_ret,out_ret

def pic_pool_history(returndata=False,save_local_file=False):

    pool_his=get_core_poolfromlocaldb()

    jjdm_con = util.list_sql_condition(pool_his['jjdm'].unique().tolist())
    value_df_list, industry_df_list, stock_df_list = get_jj_picture(jjdm_con)

    industry_style_new, class_list_new, style_type_dis_new, style_incline_dis_new, \
    style_weight_dis_new, style_type_dis2_new, style_incline_dis2_new, style_weight_dis2_new, \
    stock_style_a_new, stock_style_b_new, stock_left_new, stock_new_new, stock_fin_new=pd.DataFrame(),[pd.DataFrame(),pd.DataFrame(),pd.DataFrame()],\
                                                                                       pd.DataFrame(),pd.DataFrame(),\
                                                                                       pd.DataFrame(),pd.DataFrame(),pd.DataFrame(),\
                                                                                       pd.DataFrame(),pd.DataFrame(),pd.DataFrame(),\
                                                                                       pd.DataFrame(),pd.DataFrame(),pd.DataFrame()

    new_col_list = []
    for asofdate in pool_his['asofdate'].unique():
        jjdm_df=pool_his[pool_his['asofdate']==asofdate]['jjdm'].to_frame('jjdm_x')
        new_col_list.append(str(asofdate))
        value_df_list_temp=[]
        industry_df_list_temp=[]
        stock_df_list_temp=[]
        for i in range(len(value_df_list)):
            value_df_list_temp.append(pd.merge(jjdm_df,value_df_list[i],
                                           how='left',left_on='jjdm_x',right_on='jjdm').drop('jjdm_x',axis=1))
        for i in range(len(industry_df_list)):
            industry_df_list_temp.append(pd.merge(jjdm_df,industry_df_list[i],
                                           how='left',left_on='jjdm_x',right_on='jjdm').drop('jjdm_x',axis=1))
        for i in range(len(stock_df_list)):
            stock_df_list_temp.append(pd.merge(jjdm_df,stock_df_list[i],
                                           how='left',left_on='jjdm_x',right_on='jjdm').drop('jjdm_x',axis=1))

        industry_style, class_list, style_type_dis, style_incline_dis, \
        style_weight_dis, style_type_dis2, style_incline_dis2, style_weight_dis2, \
        stock_style_a, stock_style_b, stock_left, stock_new, stock_fin\
            =features_calculation(value_df_list_temp, industry_df_list_temp, stock_df_list_temp)

        industry_style_new = pd.merge(industry_style_new,industry_style,  how='outer', left_index=True,
                                      right_index=True)


        for i in range(3):

            class_list[i].columns = [x+"_"+str(asofdate) for x in class_list[i].columns]
            class_list_new[i] = pd.merge( class_list_new[i],class_list[i], how='outer',
                                         left_index=True, right_index=True)
            class_list_new[i] = class_list_new[i][class_list_new[i].columns.sort_values()]

        style_type_dis_new = pd.merge( style_type_dis_new,style_type_dis, how='outer',
                                      left_index=True, right_index=True).fillna(0)


        style_incline_dis_new = pd.merge(style_incline_dis_new,style_incline_dis, how='outer',
                                         left_index=True, right_index=True).fillna(0)


        style_weight_dis_new = pd.merge( style_weight_dis_new,style_weight_dis, how='outer',
                                        left_index=True, right_index=True).fillna(0)


        style_type_dis2_new = pd.merge(style_type_dis2_new,style_type_dis2,  how='outer',
                                       left_index=True, right_index=True).fillna(0)


        style_incline_dis2_new = pd.merge( style_incline_dis2_new,style_incline_dis2,
                                          how='outer', left_index=True, right_index=True).fillna(0)


        style_weight_dis2_new = pd.merge(style_weight_dis2_new,style_weight_dis2,
                                         how='outer', left_index=True, right_index=True).fillna(0)


        stock_style_a_new = pd.merge(stock_style_a_new,stock_style_a,
                                     how='outer', left_index=True, right_index=True).fillna(0)


        stock_style_b_new = pd.merge( stock_style_b_new,stock_style_b, how='outer'
                                     , left_index=True, right_index=True).fillna(0)


        stock_left_new = pd.merge( stock_left_new,stock_left, how='outer'
                                  , left_index=True, right_index=True).fillna(0)


        stock_new_new = pd.merge(stock_new_new, stock_new, how='outer',
                                 left_index=True, right_index=True).fillna(0)


        stock_fin_new = pd.merge(stock_fin_new,stock_fin,  how='outer',
                                 left_index=True, right_index=True).fillna(0)


    industry_style_new.columns = new_col_list
    style_type_dis_new.columns = new_col_list
    style_incline_dis_new.columns = new_col_list
    style_weight_dis_new.columns = new_col_list
    style_type_dis2_new.columns = new_col_list
    style_incline_dis2_new.columns =new_col_list
    style_weight_dis2_new.columns = new_col_list
    stock_style_a_new.columns = new_col_list
    stock_style_b_new.columns = new_col_list
    stock_left_new.columns = new_col_list
    stock_new_new.columns = new_col_list
    stock_fin_new.columns = new_col_list


    pool_picturing_engine(industry_style_new, class_list_new, style_type_dis_new, style_incline_dis_new, \
    style_weight_dis_new, style_type_dis2_new, style_incline_dis2_new, style_weight_dis2_new, \
    stock_style_a_new, stock_style_b_new, stock_left_new, stock_new_new, stock_fin_new,save_local_file=save_local_file)


    if(returndata):
        return industry_style_new, class_list_new, style_type_dis_new, style_incline_dis_new, \
    style_weight_dis_new, style_type_dis2_new, style_incline_dis2_new, style_weight_dis2_new, \
    stock_style_a_new, stock_style_b_new, stock_left_new, stock_new_new, stock_fin_new

def get_pool_history_report():
    
    def style_analysis(df, additional_str1='',additional_str2=''):
        style_style = "从基金{0}来看，其中:".format(additional_str1)
        for style in df.index:
            style_style += "{0}，".format(str(style)+additional_str2)
            style_style+="历史最高为{0}，历史最低为{1}。最新一期为{2}，最早一期为{3}"\
                .format((str(round(df.loc[style].max(), 2)) + "%,"),(str(round(df.loc[style].min(), 2)) + "%,"),
                        (str(round(df.loc[style].iloc[-1], 2)) + "%,"),(str(round(df.loc[style].iloc[0], 2)) + "%,"))

        return style_style

    pic_name_list = ['行业风格分布.png', '1级行业持仓分布_占持仓比例.png',
                     '1级行业持仓分布_占持仓比例排名.png',
                     '2级行业持仓分布_占持仓比例.png',
                     '2级行业持仓分布_占持仓比例排名.png',
                     '3级行业持仓分布_占持仓比例.png',
                     '3级行业持仓分布_占持仓比例排名.png',
                     '风格类型分布.png',
                     '风格专注型分布.png',
                     '成长价值持仓占比.png',
                     '成长价值持仓占比排名.png',
                     '规模风格类型分布.png',
                     '专注型风格分布.png',
                     '大中小盘持仓占比.png',
                     '大中小盘持仓占比排名.png',
                     '个股风格类型分布A.png',
                     '个股风格类型分布B.png',
                     '左侧类型分布.png',
                     '新股偏好分布.png',
                     '个股持仓特征.png',
                     ]

    industry_style, class_list, style_type_dis, style_incline_dis, \
    style_weight_dis, style_type_dis2, style_incline_dis2, style_weight_dis2, \
    stock_style_a, stock_style_b, stock_left, stock_new, stock_fin =\
        pic_pool_history(returndata=True,save_local_file=True)


    time_length=int(len(class_list[0].columns)/2)
    report_date=class_list[0].columns[0:time_length]
    report_date_rank=class_list[0].columns[time_length:]

    data_list=[industry_style , class_list[0][report_date],class_list[0][report_date_rank],
               class_list[1][report_date],class_list[1][report_date_rank],
               class_list[2][report_date],class_list[2][report_date_rank], style_type_dis , style_incline_dis , \
    style_weight_dis[0:2] ,style_weight_dis.iloc[2:], style_type_dis2 , style_incline_dis2 , style_weight_dis2[0:3] ,style_weight_dis2[3:], \
    stock_style_a , stock_style_b , stock_left , stock_new , stock_fin]
    key_word_list=['行业配置类型','一级行业分布','一级行业分布排名','二级行业分布','二级行业分布排名','三级行业分布','三级行业分布排名',
                   '风格类型','中属于风格专注的基金','持仓中的风格占比','持仓中的风格占比排名','规模风格类型','中属于风格专注的基金','持仓中的规模风格占比','持仓中的规模风格占比排名',
                   '个股配置风格','个股策略','个股左侧特征','对次新股的偏好分布','其他个股持仓特征']
    key_word_list2 = ['型占比', '行业占比', '行业占比相对排名', '行业占比', '行业占比相对排名',
                      '行业占比', '行业占比相对排名','型占比', '型占比','占比','排名','型占比','型占比',
                      '占比','排名','型占比','型占比','型占比','型占比','']

    doc = Document()
    fold = r"E:\GitFolder\hbshare\fe\mutual_analysis"

    title_num=2
    name=''
    pic_paragraphs_list = []
    string_paragraphs_list = []
    paragraphs_count = 0

    doc.add_paragraph("")
    paragraphs_count += 1

    p = doc.add_paragraph("按照季度，对公墓核心池历史进行了统计分析，结果如下")
    p.style.font.size = Pt(10)
    p.paragraph_format.first_line_indent = p.style.font.size * 2
    string_paragraphs_list.append(paragraphs_count)
    paragraphs_count += 1

    for i in range(len(pic_name_list)):
        pic=pic_name_list[i]
        file_path = fold + "\\" + pic
        desc = style_analysis(data_list[i].fillna(0), key_word_list[i],key_word_list2[i])

        doc.add_paragraph("")
        paragraphs_count += 1

        try:
            doc.add_picture(file_path, width=Inches(6), height=Inches(4))
            pic_paragraphs_list.append(paragraphs_count)
            paragraphs_count += 1

        except Exception as e:
            pic_temp = Image.open(file_path)
            pic_temp.save(pic_temp)
            doc.add_picture(file_path, width=Inches(6), height=Inches(4))

        p = doc.add_paragraph(desc)
        p.style.font.size = Pt(10)
        p.paragraph_format.first_line_indent = p.style.font.size * 2
        string_paragraphs_list.append(paragraphs_count)
        paragraphs_count += 1

    for j in pic_paragraphs_list:
        doc.paragraphs[j].alignment = WD_PARAGRAPH_ALIGNMENT.CENTER
    for j in string_paragraphs_list:
        doc.paragraphs[j].alignment = WD_PARAGRAPH_ALIGNMENT.JUSTIFY

    doc.save(r"E:\GitFolder\hbshare\fe\mutual_analysis\公募核心池历史报告.docx")
    print('')


if __name__ == '__main__':

    get_pool_history_report()
    #pool_his_fromexcel2db()
    #pic_pool_history()
    #pool_comparision('202103', '202203')

    #pool_picturing


import hbshare as hbs
from peewee import *


def init_hbs():
	hbs.set_token('Howbuy!')


record_db = DatabaseProxy()


class NetWorthRecord(Model):
	index_date = DateTimeField()
	index_value = FloatField()
	fund_manager_code = CharField()
	record_date = DateTimeField()
	record_type = CharField()
	mix_fund_latest_announcement = TextField()

	class Meta:
		database = record_db


class ShareIndex(Model):
	manager_code = CharField()
	index_date = DateTimeField()
	index_value = FloatField()

	class Meta:
		database = record_db


class BondIndex(Model):
	manager_code = CharField()
	index_date = DateTimeField()
	index_value = FloatField()

	class Meta:
		database = record_db




# -*- coding: utf-8 -*-


class BaseError(Exception):
    def __init__(self, message=''):
        Exception.__init__(self, '%s' % message)


class AlgoError(BaseError):
    def __init__(self, message=''):
        BaseError.__init__(self, message)


class PreprocessError(BaseError):
    def __init__(self, message=''):
        BaseError.__init__(self, message)


class InputParameterError(BaseError):
    def __int__(self, message='invalid input parameter'):
        BaseError.__init__(self, message)


class InputParameterWarning(BaseError):
    def __int__(self, message='invalid input parameter'):
        BaseError.__init__(self, message)



import pandas as pd
import datetime
import plotly.graph_objs as go
import  plotly.figure_factory as ff
import statsmodels.api as sm
import numpy as np
from hbshare.fe.XZ import db_engine
import plotly
import plotly.io as pio
import cvxopt.solvers as sol
from cvxopt import matrix as mat
sol.options['show_progress'] = False

hbdb=db_engine.HBDB()

class Untils:

    def __init__(self,val_hld=None,indus_info=None,fin_info=None,benchmark_info=None):
        from hbshare.fe.XZ.config import config_pfa
        configuration = config_pfa.Config()
        self.asset_type_code= configuration.Asset_type
        self.val_hld=val_hld
        self.indus_info=indus_info
        self.fin_info=fin_info
        self.bench_info=benchmark_info
        if((val_hld is not None)&(indus_info is not None)&(fin_info is not None)&(benchmark_info is not None)):
            self.purified_stk_hld= self.cleaning_stock_hld()

    def shift_df_date(self,bench_df,df,bench_date_col,date_col):

        for date in list(set(bench_df[bench_date_col].unique()).difference(set(df[date_col].unique()))):
            date_delta=df[date_col]-date
            df .loc[date_delta.abs() == date_delta.abs().min(), date_col] = date
        return df

    def cleaning_stock_hld(self):

        rawdata=self.val_hld[~self.val_hld['Stock_code'].isnull()]

        temp_df=pd.merge(
            rawdata,self.indus_info[['SECUCODE','FIRSTINDUSTRYNAME']],how='left',left_on='Stock_code',right_on='SECUCODE'
        ).drop(['SECUCODE'],axis=1)

        #original used to shift the date that is missed in from any information table(any table except the fund valuation table)
        #this part is then abandoned by using '' for missing date
        # self.fin_info=self.shift_df_date(temp_df,self.fin_info,'Stamp_date','TRADINGDAY')
        # self.bench_info = self.shift_df_date(temp_df, self.bench_info, 'Stamp_date', 'ENDDATE')[['SECUCODE','WEIGHT','ENDDATE','Index_type']]

        self.bench_info=self.bench_info.rename(columns={'WEIGHT':'Index_Weight'})
        self.bench_info=pd.merge(temp_df[['Stamp_date', 'Stock_code','Weight']],self.bench_info,how='left',left_on=['Stock_code','Stamp_date'],right_on=['SECUCODE','ENDDATE'])
        self.bench_info['Index_type'].fillna('1800以外', inplace=True)
        self.bench_info['Index_Weight'].fillna(0, inplace=True)
        temp_df=pd.merge(temp_df,self.fin_info,how='left',left_on=['Stock_code','Stamp_date'],right_on=['SECUCODE','TRADINGDAY']).drop('ROW_ID',axis=1)

        return temp_df

    def aggregate_by(self,df,groupby,method,method_on):

        if(method=='sum'):
            output_df = df.groupby(groupby).sum(method_on).unstack().fillna(0)[
                (method_on)].fillna(0)
        elif(method=='average'):
            output_df = df.groupby(groupby).mean(method_on).unstack().fillna(0)[
                (method_on)].fillna(0)
        else:
            raise Exception
            print('Please check the aggregation method')

        output_df['日期'] = output_df.index

        return output_df

    def asset_allocation_stats(self):

        data=self.val_hld
        output_df = pd.DataFrame(columns=['日期'])
        output_df['日期']=data['Stamp_date'].unique()

        for keys in self.asset_type_code.keys():

            output_df =pd.merge(output_df,data[data['Code']==self.asset_type_code[keys]][['Weight','Stamp_date']],
                                how='left',left_on='日期',right_on='Stamp_date')
            output_df.rename(columns={'Weight':keys},inplace=True)
            output_df=output_df.drop(['Stamp_date'],axis=1)

        output_df['A股']=0
        for col in [x for x in list(self.asset_type_code.keys()) if ('上交所' in x or '深交所' in x) ]:
            output_df['A股']= output_df['A股']+output_df[col].fillna(0)
        output_df['港股']=output_df['港股通'].fillna(0)+output_df['深港通'].fillna(0)

        return output_df.fillna(0)

    def rank_filter(self,input_df,thresholds):

        index_list=['前'+str(x)+'大' for x in thresholds ]
        output_df=pd.DataFrame(columns=input_df['日期'])
        input_df=input_df.drop(['日期'],axis=1).T
        for col in input_df.columns:
            values=[]
            for rank in thresholds:
                values.append( [ input_df[col].sort_values(ascending=False).values[0:rank],
                                 input_df[col].sort_values(ascending=False).index[0:rank]] )
            output_df[col]=values
        output_df.index=index_list
        output_df=output_df.T
        output_df['日期']=output_df.index

        return  output_df

    def fund_risk_exposure(self,left_table,row_factors_df,left_col):

        left_table['Stamp_date']=[ ''.join(x.split('-')) for x in left_table[left_col[0]].astype(str)]
        factors_col=row_factors_df.columns.drop(['ticker','trade_date']).tolist()
        fund_factors=pd.merge(left_table,row_factors_df
                              ,how='left',right_on=['ticker','trade_date'],left_on=[left_col[1],'Stamp_date'])\
            .drop(left_col[1],axis=1)

        for col in factors_col:
            fund_factors[col]=fund_factors[col].astype(float)*fund_factors[left_col[2]]/100
        fund_factors=fund_factors.groupby(['trade_date']).sum(factors_col)[factors_col]

        # fund_factors['Stamp_date'] = fund_factors.index
        fund_factors['JYYF']=[ x[0:6] for x in fund_factors.index]

        return fund_factors

    def generate_ret_df(self):


        ret_df=self.val_hld[self.val_hld['Code'].str.contains('今日单位净值') | self.val_hld['Code'].str.contains('基金单位净值:')]['Code'].unique()[0]
        ret_df=self.val_hld[self.val_hld['Code']==ret_df][['Code','Name','Stamp_date']]
        ret_df.rename(columns={'Name':'Net_value'},inplace=True)
        ret_df['Return']=ret_df['Net_value'].astype(float).pct_change()
        ret_df.drop('Code',axis=1,inplace=True)
        ret_df.reset_index(drop=True, inplace=True)

        return ret_df

    def iter_list(self,inputlist,iter_num,bench_factor):
        import itertools
        iter_list=list(itertools.combinations(inputlist,iter_num-1))
        output_list=[]
        for col in bench_factor:
            output_list+=[x+(col,) for x in iter_list]
        return output_list

    def calculate_alpha(self,ols_df,factors_list,num_factors,ret_col,date_col,bench_factor):

        iter_list = self.iter_list(factors_list, num_factors,bench_factor)
        alpha = []
        rsquar = []
        factors = []
        parameters = []
        for factors_combine in iter_list:
            model = sm.OLS(ols_df[ret_col].values, ols_df[['const'] + list(factors_combine)].values)
            results = model.fit()
            alpha.append(results.params[0])
            rsquar.append(results.rsquared)
            factors.append(list(factors_combine))
            parameters.append(results.params)

        summary_df = pd.DataFrame()
        summary_df['alpha'] = alpha
        summary_df['rsquar'] = rsquar
        summary_df['factors'] = factors
        summary_df['parameters'] = parameters

        max_rsquare = summary_df['rsquar'].max()
        max_rsquare_para_value = summary_df[summary_df['rsquar'] == max_rsquare]['parameters'].values[0]
        max_rsquare_para_name = ['const'] + summary_df[summary_df['rsquar'] == max_rsquare]['factors'].values[0]

        # max_rsquare_alpha=
        # max_rsquare_alpha_t_value=
        # max_rsquare=1/max_rsquare
        # alpha_score=0.25*max_rsquare_alpha

        simulated_df = ols_df[max_rsquare_para_name]
        simulated_df['simulated_ret'] = np.dot(simulated_df, max_rsquare_para_value.T)
        simulated_df['real_ret'] = ols_df[ret_col].values
        simulated_df['日期'] = ols_df[date_col].values

        return summary_df,simulated_df,','.join(max_rsquare_para_name)

    @staticmethod
    def list_sql_condition(list):
        return "'"+"','".join(list)+"'"

    @staticmethod
    def get_mutual_stock_funds(asofQ):

        hbdb=db_engine.HBDB()

        last_2years=(datetime.datetime.strptime(asofQ, '%Y%m%d')-datetime.timedelta(days=730))\
            .strftime('%Y%m%d')

        sql="select jjdm,jjmc,clrq  from st_fund.t_st_gm_jjxx where cpfl='2' and (ejfl='13' or ejfl='35' or ejfl='37') and jjzt='0' and clrq<='{0}'  ".format(last_2years)
        stock_jjdm=hbdb.db2df(sql,db='funduser').sort_values(['clrq','jjdm'])
        stock_jjdm.drop_duplicates(['jjmc'],keep='first',inplace=True)

        jjdm_list=stock_jjdm['jjdm'].tolist()
        ticker_con="'"+"','".join(jjdm_list)+"'"

        sql="select jjdm,avg(gptzzjb) from st_fund.t_st_gm_zcpz where jjdm in ({0}) group by jjdm"\
            .format(ticker_con)
        tempdf = hbdb.db2df(sql, db='funduser')
        jjdm_list=tempdf[tempdf['avg(gptzzjb)']>=60]['jjdm'].tolist()
        ticker_con = "'" + "','".join(jjdm_list) + "'"

        sql="select jjdm,min(gptzzjb) from st_fund.t_st_gm_zcpz where jsrq<='{0}' and jsrq>={1} and jjdm in ({2}) group by jjdm"\
            .format(asofQ,last_2years,ticker_con)

        tempdf=hbdb.db2df(sql,db='funduser')
        jjdm_list = tempdf[tempdf['min(gptzzjb)'] >= 50]['jjdm'].tolist()

        return  jjdm_list

    @staticmethod
    def _shift_date(date):
        trade_dt = datetime.datetime.strptime(date, '%Y%m%d')
        pre_date = (trade_dt -datetime.timedelta(days=30)).strftime('%Y%m%d')

        sql_script = "SELECT JYRQ, SFJJ, SFYM FROM funddb.JYRL WHERE JYRQ >= {} and JYRQ <= {}".format(
            pre_date,date)
        df=hbdb.db2df(sql_script,db='readonly')
        df=df.rename(
            columns={"JYRQ": 'calendarDate', "SFJJ": 'isOpen',
                      "SFYM": "isMonthEnd"}).sort_values(by='calendarDate')
        df['isOpen'] = df['isOpen'].astype(int).replace({0: 1, 1: 0})
        df['isMonthEnd'] = df['isMonthEnd'].fillna(0).astype(int)

        trading_day_list = df[df['isOpen'] == 1]['calendarDate'].tolist()

        return trading_day_list[-1]

    @staticmethod
    # 写一个通用的多元一次回归的算法模型，自变量个数不确定
    def my_general_linear_model_func(A1, b1):

        P = 2 * np.dot(np.transpose(A1), A1)
        Q = - 4 * np.dot(np.transpose(A1), b1)
        P = mat(P)
        Q = mat(Q)

        lb=[0]*A1.shape[1]
        ub=[1]*A1.shape[1]

        G = mat(np.vstack((np.diag([-1]*A1.shape[1]), np.diag([1]*A1.shape[1]))), tc='d')
        h = mat(np.array(lb+ub), tc='d') # 为各参数的上下限！！！！
        A = mat(np.array([[1]*A1.shape[1]]), tc='d')
        b = mat(np.array([1]), tc='d')

        result = sol.qp(P, Q,G, h, A, b)

        return [ x for x in result['x']]

class Plot:
    def __init__(self,fig_width,fig_height):

        self.fig_width=fig_width
        self.fig_height=fig_height
        self.ams_color_lista=['#C94649','#EEB2B4','#E1777A','#D57C56','#E39A79','#DB8A66','#E5B88C']
        self.ams_color_listb = ['#8588B7', '#B4B6D1', '#55598D', '#628497', '#A9C6CB','#866EA9','#B79BC7']
        self.ams_color_listc = ['#7D7D7E', '#CACACA', '#A7A7A8', '#606063', '#C4C4C4','#99999B','#B7B7B7']
    #
    def plot_render(self,data,layout, **kwargs):
        kwargs['output_type'] = 'div'
        plot_str = plotly.offline.plot({"data": data, "layout": layout}, **kwargs)
        print('%%angular <div style="height: %ipx; width: %spx"> %s </div>' % (self.fig_height, self.fig_width, plot_str))

    # def plot_render(self,data,layout):
    #     fig = go.Figure(data=data, layout=layout)
    #     fig.show()

    @staticmethod
    def save_pic2local(data,layout,name):
        fig = go.Figure(data=data, layout=layout)
        fig.write_image(name+'.png')
        # pio.write_image(fig,name+'.png')


    def plotly_style_bar(self,df, title_text,legend_x=0.30):
        fig_width, fig_height = self.fig_width,self.fig_height
        cols = df.columns.tolist()

        data = []
        for i in range(len(cols)):
            col = cols[i]
            trace = go.Bar(
                x=df.index.tolist(),
                y=df[col],
                name=col,

            )
            data.append(trace)

        layout = go.Layout(
            title=dict(text=title_text),
            autosize=False, width=fig_width, height=fig_height,
            yaxis=dict(tickfont=dict(size=12), showgrid=True),
            xaxis=dict(showgrid=True),
            legend=dict(orientation="h", x=legend_x),
            template='plotly_white'
        )

        self.plot_render(data, layout)

    def plotly_jjpic_bar(self,df, title_text,legend_x=0.30):
        fig_width, fig_height = self.fig_width,self.fig_height
        cols = df.columns.tolist()

        a=int(np.floor(len(cols)/3))
        b=len(cols)%3

        color_list=self.ams_color_lista[0:a+int(b==1)]+\
                   self.ams_color_listb[0:a+int(b==2)]+\
                   self.ams_color_listc[0:a]

        data = []
        for i in range(len(cols)):
            col = cols[i]
            trace = go.Bar(
                x=df.index.tolist(),
                y=df[col],
                name=col,
                marker=dict(color=color_list[i])
            )
            data.append(trace)

        layout = go.Layout(
            title=dict(text=title_text),
            autosize=False, width=fig_width, height=fig_height,
            yaxis=dict(tickfont=dict(size=12), showgrid=True),
            xaxis=dict(showgrid=True),
            legend=dict(orientation="h", x=legend_x),
            template='plotly_white'
        )

        self.plot_render(data, layout)
        #self.save_pic2local(data, layout,title_text)

    def plotly_pie(self,df, title_text):
        fig_width, fig_height = self.fig_width,self.fig_height
        labels = df.index.tolist()
        values = df.values.round(3).reshape(1,len(df)).tolist()[0]

        a=int(np.floor(len(labels)/3))
        b=len(labels)%3

        color_list=self.ams_color_lista[0:a+int(b==1)]+\
                   self.ams_color_listb[0:a+int(b==2)]+\
                   self.ams_color_listc[0:a]

        # color_list = ['#C94649', '#8588B7', '#7D7D7E']
        data = [go.Pie(labels=labels, values=values, hoverinfo="label+percent",marker=dict(colors=color_list),
                       texttemplate="%{label}: %{percent}")]
        layout = go.Layout(
            title=dict(text=title_text),
            autosize=False, width=fig_width, height=fig_height
        )

        self.plot_render(data, layout)
        #self.save_pic2local(data, layout, title_text)

    def plotly_area(self,df,title_text):
        fig_width, fig_height = self.fig_width,self.fig_height
        names = df.columns.to_list()
        names.remove('日期')
        cols =df['日期'].to_list()

        data = []
        for name in names:
            tmp = go.Scatter(
                x=cols,
                y=df[name].values,
                name=name,
                mode='lines',
                line=dict(width=0.5),
                fill='tonexty',
                stackgroup='one')
            data.append(tmp)

        layout = go.Layout(
            title=title_text,
            autosize=False,
            width=fig_width,
            height=fig_height,
            showlegend=True,
            xaxis=dict(type='category'),
            yaxis=dict(
                type='linear',
                range=[1, 100],
                dtick=20,
                ticksuffix='%'))

        self.plot_render(data, layout)

    def plotly_line(self,df, title_text):

        fig_width, fig_height = self.fig_width,self.fig_height
        names = df.columns.to_list()
        names.remove('日期')
        cols =df['日期'].to_list()

        data = []
        for name in names:
            trace = go.Scatter(
                x=cols,
                y=df[name].values,
                name=name,
                mode="lines+markers"
            )
            data.append(trace)

        date_list = df.index.tolist()

        layout = go.Layout(
            title=dict(text=title_text),
            autosize=False, width=fig_width, height=fig_height,
            yaxis=dict(tickfont=dict(size=12), showgrid=True),
            xaxis=dict(showgrid=True),
            yaxis2={'anchor': 'x', "overlaying": 'y', "side": 'right'},
            template='plotly_white'
        )

        self.plot_render(data, layout)

    def plotly_line_style(self, df, title_text,fix_range=False):

        fig_width, fig_height = self.fig_width,self.fig_height
        cols = df.columns.tolist()
        # color_list = ['rgb(49, 130, 189)', 'rgb(204, 204, 204)', 'rgb(216, 0, 18)']
        data = []

        data = []
        for col in cols:
            trace = go.Scatter(
                x=df.index.tolist(),
                y=df[col],
                name=col,
                mode="lines+markers"
            )
            data.append(trace)

        date_list = df.index.tolist()

        layout = go.Layout(
            title=dict(text=title_text),
            autosize=False, width=fig_width, height=fig_height,
            # yaxis=dict(tickfont=dict(size=12), showgrid=True),
            xaxis=dict(showgrid=True),
            # yaxis2={'anchor': 'x', "overlaying": 'y', "side": 'right'},
            template='plotly_white'
        )

        self.plot_render(data, layout)

    def plotly_line_multi_yaxis(self,df,title_text,y2_col):

        fig_width, fig_height = self.fig_width,self.fig_height
        names = df.columns.to_list()
        for name in y2_col+['日期']:
            names.remove(name)
        cols =df['日期'].to_list()

        data = []
        for name in names:
            trace = go.Scatter(
                x=cols,
                y=df[name].values,
                name=name+'(左轴)',
                mode="lines+markers"
            )
            data.append(trace)

        for name in y2_col:
            trace = go.Scatter(
                x=cols,
                y=df[name].values,
                name=name+'(右轴)',
                mode="lines+markers",
                yaxis='y2'
            )
            data.append(trace)

        date_list = df.index.tolist()

        layout = go.Layout(
            title=dict(text=title_text),
            autosize=False, width=fig_width, height=fig_height,
            yaxis=dict(tickfont=dict(size=12), showgrid=True),
            xaxis=dict(showgrid=True),
            yaxis2={'anchor': 'x', "overlaying": 'y', "side": 'right'},
            template='plotly_white'
        )

        self.plot_render(data, layout)

    def plotly_table(self, df, table_width, title_text):

        fig=ff.create_table(df)
        fig.layout.width=table_width
        fig.layout.title=title_text
        self.plot_render(fig.data,fig.layout )

    def ploty_polar(self,df,title):

        fig_width, fig_height = self.fig_width, self.fig_height

        th=df.columns.tolist()
        r_data=df.values[0].tolist()

        trace0 = go.Scatterpolar(
            r=r_data,
            theta=th,
            fill='toself',
        )

        data = [trace0]
        layout = go.Layout(
            title=dict(text=title),
            autosize=False, width=fig_width, height=fig_height,
            polar=dict(
                radialaxis=dict(visible=True,range = [0, 9])
                ),
            showlegend=False
        )

        self.plot_render(data, layout)

    def ploty_heatmap(self,z,z_text,title):

        #fig = make_subplots(rows=3, cols=1, shared_xaxes=True, vertical_spacing=0.03, row_width=[0.3, 0.2, 0.7])

        fig_width, fig_height = self.fig_width, self.fig_height

        fig=ff.create_annotated_heatmap(z,annotation_text=z_text)
        fig['layout']['yaxis']['autorange'] = "reversed"

        fig.layout.width=fig_width
        fig.layout.title=title

        self.plot_render(fig.data,fig.layout)

    def plotly_line_with_annotation(self, df,data_col,anno_col, title_text,fix_range=False):

        fig_width, fig_height = self.fig_width,self.fig_height
        cols =data_col

        data = []
        for col in cols:
            trace = go.Scatter(
                x=df.index.tolist(),
                y=df[col].values.tolist(),
                name=col,
                mode="lines"
            )
            data.append(trace)
        df=df[df[anno_col[0]].notnull()]
        for col2 in anno_col:
            trace=go.Annotation(
                textfont=go.Font(
                    size=1
                ),
                text=df[col2].values.tolist(),
                x=df.index.tolist(),
                y=df[col].values.tolist(),
                mode="markers",
                marker=dict( size=10),
                name='trade_point',

            )
            data.append(trace)

        # date_list = df.index.tolist()


        layout = go.Layout(
            title=dict(text=title_text),
            autosize=False, width=fig_width, height=fig_height,
            # yaxis=dict(tickfont=dict(size=12), showgrid=True),
            xaxis=dict(showgrid=True),
            # yaxis2={'anchor': 'x', "overlaying": 'y', "side": 'right'},
            template='plotly_white',


        )

        self.plot_render(data, layout)


if __name__ == '__main__':

    uti=Untils()
    uti.get_mutual_stock_funds('20211231')


#!/usr/bin/python
#coding:utf-8

"""
@author: Meng.lv
@contact: meng.lv@howbuy.com
@software: PyCharm
@file: __init__.py.py
@time: 2020/6/15 10:04
"""


def main():
    pass


if __name__ == "__main__":
    main()


sql_quote = '''(
    `ID` bigint not null COMMENT \'主键\' AUTO_INCREMENT,
    `TDATE`  date not null COMMENT \'交易日期\',
    `EXCHANGE` varchar(255) COMMENT \'交易所代码\',
    `SYMBOL` varchar(255) COMMENT \'交易代码\',
    `LCLOSE` float COMMENT \'前收盘价(元)\',
    `OPEN` float COMMENT \'开盘价(元)\',
    `HIGH` float COMMENT \'最高价(元)\',
    `LOW` float COMMENT \'最低价(元)\',
    `CLOSE` float COMMENT \'收盘价(元)\',
    `VOLUME` bigint COMMENT \'成交量(股)\',
    `AMOUNT` bigint COMMENT \'成交金额(元)\',
    `AVGPRICE` float COMMENT \'当日成交均价(元)\',
    `CHG` float COMMENT \'涨跌金额(元)\', 
    `PCHG` float COMMENT \'涨跌幅(%)\', 
    `PRANGE` float COMMENT \'振幅(%)\', 
    `MCAP` bigint COMMENT \'流通市值(元)\', 
    `TCAP` bigint COMMENT \'总市值(元)\', 
    primary key (`ID`)
    )
    '''

# 国泰君安估值表列
gtja_gzb_col_sql = '''(
    `id` bigint not null AUTO_INCREMENT,
    `日期` date,
    `基金名称`  varchar(100),
    `科目代码`  varchar(100),
    `科目名称`  varchar(100),
    `数量`  double,
    `单位成本`  float,
    `成本`  double,
    `成本占净值`  float,
    `市价`  float,
    `市值`  double,
    `市值占净值`  float,
    `估值增值`  double,
    `停牌信息`  varchar(100),
    primary key (`id`)
    )
    '''




# -*- coding: utf-8 -*-

from hbshare.fe.xwq.analysis.utils.const_var import TimeDateFormat
from hbshare.fe.xwq.analysis.utils.logger_utils import LoggerUtils
from hbshare.fe.xwq.analysis.utils.timedelta_utils import TimeDateUtil
import pandas as pd
import os


class Preprocess:
    @staticmethod
    def check_newest_data(factor_path, factor_names=[]):
        """
        判断各个因子的最新时间
        :param factor_path:
        :param factor_names:
        :return:
        """
        newest_date = ''
        for factor_name in factor_names:
            td = Preprocess.check_lastest_date(factor_path, factor_name.lower())
            newest_date = td if newest_date == '' or td == '' or td < newest_date else newest_date
        return newest_date

    @staticmethod
    def check_lastest_date(factor_path, factor_name):
        """
        检验给定某因子，在因子文件夹路径下的最新日期
        :param factor_path:
        :param factor_name:
        :return:
        """
        dir_path = factor_path + factor_name
        newest_date = ''
        if not os.path.isdir(dir_path):
            return newest_date
        for f_name in os.listdir(dir_path):
            items = f_name.split('.')
            if not os.path.isfile(dir_path + '/' + f_name) or factor_name not in f_name or len(items) != 3:
                continue
            date = TimeDateUtil.convert_format(items[1], TimeDateFormat.YMD.value, TimeDateFormat.YMDHYPHEN.value)
            newest_date = date if newest_date == '' or date > newest_date else newest_date
        return newest_date

    @staticmethod
    def output_factor(factor_path, factor_name, start_date, end_date, factor_df):
        """
        输出因子文件
        :return:
        """
        if factor_name not in factor_df.columns:
            LoggerUtils().get_logger().error('[Preprocess][OutputFactor] factorName:{0} not in df'.format(factor_name))
            return
        part_df = factor_df[['TRADE_DATE', 'TICKER-SYMBOL', factor_name]].copy(deep=True)
        part_df = part_df.dropna(subset=[factor_name], how='any', axis=0)
        part_df = part_df[(part_df['TRADE_DATE'] >= TimeDateUtil.convert_format(start_date, TimeDateFormat.YMDHYPHEN.value, TimeDateFormat.YMD.value)) &
                          (part_df['TRADE_DATE'] <= TimeDateUtil.convert_format(end_date, TimeDateFormat.YMDHYPHEN.value, TimeDateFormat.YMD.value))]
        g = part_df.groupby(['TRADE_DATE'])
        factor_name = factor_name.lower()
        for td, pp_df in g:
            dir_name = "{0}{1}".format(factor_path, factor_name)
            if not os.path.isdir(dir_name):
                os.mkdir(dir_name)
            f_name = "{0}{1}/{2}.{3}.csv".format(factor_path, factor_name, factor_name, td)
            pp_df[['TICKER_SYMBOL', factor_name.upper()]].to_csv(f_name, encoding='utf-8', header=False, index=False)
        return

    @staticmethod
    def get_neued_signal(x, previous_date, factor_name='value'):
        """
        中性化
        """
        exclude_style_factor = ['BETA', 'RESVOL']
        signal = x.copy(deep=True).dropna()
        if signal.shape[0] > 0:
            series = signal[['ticker', factor_name]].set_index('ticker').dropna()
            # neued = uqer.standardize(uqer.neutralize(uqer.winsorize(series)), target_date=previous_date,
            #                          exclude_style_list=exclude_style_factor)
            neued = uqer.standardize(uqer.neutralize(uqer.winsorize(series)))
            return neued

    @staticmethod
    def merge_dfs(dfs, aligned_cols=[]):
        """
        按给定的列名，合并多个dataframe
        :param dfs:
        :param aligned_cols:
        :return:
        """
        dfs = [df for df in dfs if len(df.index) > 0]
        merged = pd.DataFrame()
        if len(dfs) == 0:
            return merged
        merged = dfs[0]
        for i in range(1, len(dfs)):
            merged = merged.merge(dfs[i], left_on=aligned_cols, right_cols=aligned_cols, how='outer')
        return merged




import datetime
import pandas as pd
import numpy as np
from hbshare.fe.XZ import db_engine
from hbshare.fe.XZ import functionality
from sklearn.cluster import KMeans
import matplotlib.pyplot as plt
import seaborn as sns  #Python library for Vidualization
from sklearn.preprocessing import StandardScaler
from sklearn.decomposition import PCA
from sklearn.cluster import AgglomerativeClustering
from scipy.cluster.hierarchy import linkage, dendrogram
from hbshare.fe.mutual_analysis import  holdind_based as hb


util=functionality.Untils()
hbdb=db_engine.HBDB()
localdb=db_engine.PrvFunDB().engine
theme_map=hb.Industry_analysis().ind2thememap

new_col_name = ['农林牧渔', '基础化工', '钢铁', '有色金属', '电子', '家用电器', '食品饮料',
                     '纺织服饰', '轻工制造', '医药生物', '公用事业', '交通运输', '房地产', '商贸零售',
                     '社会服务', '综合', '建筑材料', '建筑装饰', '电力设备', '国防军工', '计算机', '传媒',
                     '通信', '银行', '非银金融', '汽车', '机械设备', '煤炭', '石油石化', '环保', '美容护理']
index_code_list = ['801010', '801030', '801040', '801050', '801080', '801110',
                        '801120', '801130', '801140', '801150', '801160', '801170',
                        '801180', '801200', '801210', '801230', '801710', '801720',
                        '801730', '801740', '801750', '801760', '801770', '801780',
                        '801790', '801880', '801890', '801950', '801960', '801970', '801980']
index_code_map = dict(zip(index_code_list, new_col_name))

def read_industry_info():

    #read financial info from local db
    sql="SELECT * from hbs_industry_financial_stats where class_level=1"
    industry_fin_df=pd.read_sql(sql,con=localdb)

    start_date=str(int(industry_fin_df['ENDDATE'].max()[0:4])-1)+industry_fin_df['ENDDATE'].max()[4:]+"01"
    end_date=industry_fin_df['ENDDATE'].max()+"31"
    industry_fin_df=industry_fin_df[(industry_fin_df['ENDDATE']>=start_date)&(industry_fin_df['ENDDATE']<=end_date)]

    style = util.list_sql_condition(index_code_list)
    sql = "select zqdm,jyrq,spjg from st_market.t_st_zs_hqql where zqdm in ({0}) and jyrq>='{1}' and  jyrq<='{2}'  " \
        .format(style, start_date, end_date)
    fac_ret_df = hbdb.db2df(sql, db='alluser')
    fac_ret_df['jyrq'] = fac_ret_df['jyrq'].astype(str)
    fac_ret_df.set_index('jyrq', drop=True, inplace=True)

    fac_ret_df['price'] = fac_ret_df['spjg']

    fac_ret_df['zqdm'] = [index_code_map[x] for x in fac_ret_df['zqdm']]

    fac_ret_df['ym']=fac_ret_df.index.astype(str).str[0:6]

    fac_ret_df2=fac_ret_df.groupby(['ym','zqdm']).mean().reset_index().drop('spjg',axis=1)
    fac_ret_df2['diff']=fac_ret_df2.groupby('zqdm')['price'].diff()
    fac_ret_df2['growth']=fac_ret_df2['diff']/fac_ret_df2['price']

    fac_ret_df['diff']=fac_ret_df.groupby('zqdm')['price'].diff()
    fac_ret_df['growth']=fac_ret_df['diff']/fac_ret_df['price']
    fac_ret_df.drop(['price','spjg','ym','diff'],axis=1,inplace=True)

    industry_fin_df=pd.merge(industry_fin_df,fac_ret_df2[['ym', 'zqdm','growth']],
                             how='left',left_on=['industry_name','ENDDATE'],right_on=['zqdm','ym']).drop(['zqdm','ym'],axis=1)

    return industry_fin_df,fac_ret_df

def standardlize(df):

    scaler = StandardScaler()
    scaler.fit(df)
    scaled_ds = pd.DataFrame(scaler.transform(df), columns=df.columns)

    return  scaled_ds

def PCA_function(scaled_ds,pca_num):

    if(pca_num is not None):
        pca = PCA(n_components=pca_num)
        pca.fit(scaled_ds)
        PCA_ds = pd.DataFrame(pca.transform(scaled_ds))
    else:
        PCA_ds=scaled_ds

    return  PCA_ds

def pro_of_sameclass(df):

    output=pd.DataFrame()
    for col in df.columns.tolist():
        pro_list=[]
        for col2 in df.columns.tolist():
            pro_list.append((df[col2]==df[col]).sum()/len(df))
        output[col]=pro_list
    output.index=df.columns.tolist()
    return output

def K_means(df,loop_num=100,pca_num=None):

    if(len(df.columns)==1):
        scaled_ds=df
    else:
        scaled_ds = standardlize(df)

    PCA_ds=PCA_function(scaled_ds, pca_num)

    X=PCA_ds.values

    # wcss = []
    # for i in range(1, 11):
    #     kmeans = KMeans(n_clusters=i, init='k-means++', random_state=0)
    #     kmeans.fit(X)
    #     wcss.append(kmeans.inertia_)

    # plt.plot(range(1, 11), wcss)
    # plt.title('The Elbow Method')
    # plt.xlabel('no of clusters')
    # plt.ylabel('wcss')
    # plt.show()

    theme_df=pd.DataFrame()
    score_list=[]
    for i in range(loop_num):
        kmeansmodel = KMeans(n_clusters=5, init='k-means++', random_state=i)
        y_kmeans = kmeansmodel.fit_predict(X)
        score_list.append(kmeansmodel.inertia_)

        theme_df['theme_{}'.format(i)]=y_kmeans

    return  theme_df,np.mean(score_list)

def HC(df,figname,pca_num=None):

    if(len(df.columns)==1):
        scaled_ds=df
    else:
        scaled_ds = standardlize(df)

    PCA_ds=PCA_function(scaled_ds, pca_num)
    PCA_ds.index=df.index
    merg = linkage(PCA_ds, method="ward")
    plt.rcParams['font.sans-serif']='SimHei'# scipy is an algorithm of hiyerarchal clusturing
    dendrogram(merg, leaf_rotation=90,labels=df.index)
    plt.xlabel("data points")
    plt.ylabel("euclidean distance")
    # plt.show()
    plt.savefig('{}.png'.format(figname))
    plt.close()

    # X=PCA_ds.values
    #
    # theme_df=pd.DataFrame()
    #
    # hiyerartical_cluster = AgglomerativeClustering(n_clusters=5, affinity="euclidean", linkage="ward")
    # cluster = hiyerartical_cluster.fit_predict(X)
    # theme_df['theme']=cluster

    #return  theme_df

def cluster_engine(industry_fin_df,potient_featute_list):

    method1df = pd.DataFrame()
    method2df = pd.DataFrame()
    method3df = pd.DataFrame()
    method4df = pd.DataFrame()

    for i in range(len(potient_featute_list)):
        print("test number {} \n".format(i + 1))
        if(i>0):
            feature_list = [potient_featute_list[0]]+[potient_featute_list[i]]
        else:
            feature_list= [potient_featute_list[0]]

        feature_df = industry_fin_df.groupby(['industry_name']).mean()[feature_list]
        feature_df=feature_df[feature_df.isnull().sum(axis=1) == 0]

        new_industry_fin_df = pd.DataFrame()
        new_industry_fin_df['industry_name'] = industry_fin_df['industry_name'].unique().tolist()
        for date in industry_fin_df['ENDDATE'].unique()[1:]:
            new_industry_fin_df = pd.merge(new_industry_fin_df,
                                           industry_fin_df[industry_fin_df['ENDDATE'] == date][
                                               ['industry_name', 'ENDDATE'] + feature_list], how='left',
                                           on='industry_name').drop('ENDDATE', axis=1)
        new_industry_fin_df = new_industry_fin_df[new_industry_fin_df.isnull().sum(axis=1) == 0]

        result3, kmeans_score3 = K_means(new_industry_fin_df.drop('industry_name', axis=1))

        result, kmeans_score = K_means(feature_df)

        # result2 = HC(feature_df)

        HC(new_industry_fin_df.set_index('industry_name'),feature_list[-1])


        theme_map = pd.DataFrame()
        theme_map['industry_name'] = feature_df.index
        theme_map = pd.concat([theme_map, result], axis=1)

        # theme_map2 = pd.DataFrame()
        # theme_map2['industry_name'] = industry_fin_df.groupby(['industry_name']).mean().index
        # theme_map2 = pd.concat([theme_map2, result2], axis=1)
        # theme_map2['industry_name'] = theme_map2['industry_name'] + ","

        theme_map3 = pd.DataFrame()
        theme_map3['industry_name'] = new_industry_fin_df['industry_name']
        theme_map3 = pd.concat([theme_map3, result3], axis=1)

        # theme_map4 = pd.DataFrame()
        # theme_map4['industry_name'] = industry_fin_df.groupby(['industry_name']).mean().index
        # theme_map4 = pd.concat([theme_map4, result4], axis=1)
        # theme_map4['industry_name'] = theme_map4['industry_name'] + ","

        test = theme_map.T
        test.columns = test.iloc[0]
        test.drop('industry_name', axis=0, inplace=True)
        pro1 = pro_of_sameclass(test)
        pro1['feature_list']=util.list_sql_condition(feature_list)
        test = test.astype(float).corr()
        test['feature_list']=util.list_sql_condition(feature_list)

        test2 = theme_map3.T
        test2.columns = test2.iloc[0]
        test2.drop('industry_name', axis=0, inplace=True)
        pro2 = pro_of_sameclass(test2)
        pro2['feature_list']=util.list_sql_condition(feature_list)
        test2 = test2.astype(float).corr()
        test2['feature_list']=util.list_sql_condition(feature_list)


        # dy_columns = (test > 0.8).sum().sort_values(ascending=False).index.tolist()
        # theme_cluster = []
        # for col in ['社会服务', '环保', '国防军工', '电力设备', '美容护理', '公用事业']:
        #     if (col in dy_columns):
        #         industry_list = test[test[col] > 0.8].index.tolist()
        #         theme_cluster.append(util.list_sql_condition(industry_list))
        #
        # for item in theme_cluster:
        #     print(item)
        # print('Kmeans_score is {} \n'.format(kmeans_score))
        # theme_cluster.append(kmeans_score)
        method1df=pd.concat([method1df,pro1],axis=0)
        method2df = pd.concat([method2df, test], axis=0)

        # dy_columns = (test2 > 0.8).sum().sort_values(ascending=False).index.tolist()
        # theme_cluster = []
        # for col in ['社会服务', '环保', '国防军工', '电力设备', '美容护理', '公用事业']:
        #     if (col in dy_columns):
        #         industry_list = test2[test2[col] > 0.8].index.tolist()
        #         theme_cluster.append(util.list_sql_condition(industry_list))
        # for item in theme_cluster:
        #     print(item)
        # print('Kmeans_score is {} \n'.format(kmeans_score3))
        # theme_cluster.append(kmeans_score3)
        method3df = pd.concat([method3df, pro2], axis=0)
        method4df = pd.concat([method4df, test2], axis=0)

        # print(theme_map2.groupby('theme').sum())
        # method3df = pd.concat([method3df, theme_map2.groupby('theme').sum()], axis=0)


    method1df.to_excel('method1df.xlsx')
    method2df.to_excel('method2df.xlsx')
    method3df.to_excel('method3df.xlsx')
    method4df.to_excel('method4df.xlsx')
    print('done')

def method1_usinglocabdata():

    industry_fin_df,fac_ret_df=read_industry_info()
    industry_fin_df=industry_fin_df[['industry_name','ENDDATE','ROE', 'EPS', 'OPERCASHFLOWPS', 'NETPROFITGROWRATE',
       'OPERATINGREVENUEYOY', 'NETASSETPS', 'DIVIDENDRATIO', 'PE', 'PB', 'PEG',
       'AVERAGEMV','growth']]

    industry_fin_df=industry_fin_df[(industry_fin_df['industry_name']!='综合')]
    # industry_fin_df=industry_fin_df[(industry_fin_df['industry_name'] != '农林牧渔')]

    potient_featute_list=['growth','ROE','PE','DIVIDENDRATIO','NETPROFITGROWRATE','EPS','PB','PEG','OPERCASHFLOWPS']

    cluster_engine(industry_fin_df,potient_featute_list)

def read_industry_index_data(end_date=datetime.datetime.today().strftime('%Y%m%d'),time_length=2):


    zqdm_con=util.list_sql_condition(index_code_list)
    start_date=str(int(end_date[0:4])-time_length)+end_date[4:]
    sql="select zqdm,jyrq,zdfd,roe,pe,pb,gxl from st_market.t_st_zs_hqql where jyrq>='{0}' and jyrq<='{1}' and zqdm in({2})"\
        .format(start_date,end_date,zqdm_con)
    index_data=hbdb.db2df(sql,db='alluser')
    index_data['industry_name']=[index_code_map[x] for x in index_data['zqdm']]
    index_data.rename(columns={'jyrq':'ENDDATE'},inplace=True)

    return index_data.drop('zqdm',axis=1)

def method2_usingdailyretfromdb():

    industry_fin_df=read_industry_index_data()
    potient_featute_list = ['zdfd','roe','gxl','pe']
    industry_fin_df = industry_fin_df[(industry_fin_df['industry_name'] != '综合')]
    cluster_engine(industry_fin_df, potient_featute_list)


if __name__ == '__main__':

    # method1_usinglocabdata()
    # method2_usingdailyretfromdb()

    theme_map=theme_map[(theme_map['industry_name']!='社会服务')&
                        (theme_map['industry_name']!='农林牧渔')&
    (theme_map['industry_name']!='美容护理')&
    (theme_map['industry_name']!='电力设备')&
    (theme_map['industry_name']!='农林牧渔')&
    (theme_map['industry_name']!='国防军工')&
    (theme_map['industry_name']!='建筑材料')&
    (theme_map['industry_name']!='环保')&
    (theme_map['industry_name']!='公用事业')&
    (theme_map['industry_name']!='纺织服饰')&
    (theme_map['industry_name']!='房地产')&
    (theme_map['industry_name']!='医药生物')&
    (theme_map['industry_name']!='商贸零售')&
    (theme_map['industry_name']!='传媒')&
    (theme_map['industry_name']!='有色金属')&
    (theme_map['industry_name']!='建筑装饰')&
    (theme_map['industry_name']!='基础化工')&
    (theme_map['industry_name']!='石油石化')&
    (theme_map['industry_name']!='交通运输')&
    (theme_map['industry_name']!='汽车')]
    df1 = pd.read_excel(r"E:\GitFolder\hbshare\fe\mutual_analysis\行业聚类日频数据结果\method1df.xlsx"
                        ,sheet_name='Sheet2')
    df2 = pd.read_excel(r"E:\GitFolder\hbshare\fe\mutual_analysis\行业聚类日频数据结果\method3df.xlsx",
                        sheet_name='Sheet2')
    df1=pd.merge(df1,theme_map,how='left',
                 left_on='Unnamed: 0',right_on='industry_name').drop(['Unnamed: 0','industry_name'],axis=1)
    df2=pd.merge(df2,theme_map,how='left',
                 left_on='Unnamed: 0',right_on='industry_name').drop(['Unnamed: 0','industry_name'],axis=1)
    df1=df1.groupby('theme').mean()
    df2=df2.groupby('theme').mean()
    df1.to_excel(r"E:\GitFolder\hbshare\fe\mutual_analysis\行业聚类日频数据结果\method12_result.xlsx")
    df2.to_excel(r"E:\GitFolder\hbshare\fe\mutual_analysis\行业聚类日频数据结果\method34_result.xlsx")
    print('')


# -*- coding: utf-8 -*-

import json
import requests
import sys
import time
import traceback


def curl_request(url, method, data, request_type, max_times=30):
    """
    处理请求的函数
    :return:
    """
    vaild = False
    times = 0
    dic = None
    headers = {'Content-Type': 'application/json;charset=UTF-8',
               'Accept': 'application/json, text/plain, */*',
               'Accept-Language': 'zh-cn',
               'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; Win64; x64) '
                             'AppleWebKit/537.36 (KHTML, like Gecko) '
                             'Chrome/73.0.3683.86 Safari/537.36'}
    while not vaild and times < max_times:
        try:
            if method == 'GET':
                r = requests.get(url, headers=headers)
            elif method == 'POST':
                r = requests.post(url, data=json.dumps(data), headers=headers)
            elif method == 'POST JSON':
                r = requests.post(url, json=data, headers=headers)
            else:
                r = requests.put(url, json=data, headers=headers)
            print('[Strategy][{0}] response: {1}'.format(url, r.text))
            dic = json.loads(r.text, encoding='utf-8')
            valid = True
        except Exception as e:
            exc_type, exc_value, exc_trackback = sys.exc_info()
            print(repr(traceback.format_exception(exc_type, exc_value, exc_trackback)))
            print('[Strategy] curl request existed error: {0}'.format(request_type))
            valid = False
            times += 1
            time.sleep(30)
    return dic


# -*- coding:utf-8 -*-
"""
Created on 2020/6/15
@author: Meng Lv
@group: Howbuy FDC
@contact: meng.lv@howbuy.com
"""
VERSION = '1.0.1'
P_TYPE = {'http': 'http://', 'ftp': 'ftp://', 'https': 'https://'}
FORMAT = lambda x: '%.2f' % x
FORMAT4 = lambda x: '%.4f' % x
DOMAINS = {'hbcgi': 'data.howbuy.com', 'static': 'static.howbuy.com','ams':'ams.howbuy.com'}

##########################################################################
# 基金数据列名
HOWBUY_FUND_NEWEST_NAV_COLUMNS = ['jjdm', 'jzrq', 'jjjz', 'ljjz', 'hbdr', 'hb1y',
                                  'hb3y', 'hb6y', 'hbjn', 'hb1n', 'zfxz']

HOWBUY_FUND_HOLDING_COLUMNS = ['zqdm', 'zqmc', 'zjbl', 'ccdb']

HOWBUY_RD_PROFUDUCT_COLUMNS = ['code', 'name', 'fullName', 'type','typeName']

#test
##########################################################################
# 数据源URL
HOWBUY_FUND_NEWEST_NAV = "%s%s/cgi/fund/navinfoquery.json?fundCode=%s"
HOWBUY_FUND_HOLDING = '%s%s/??/upload/auto/script/fund/data_%s.js?v=%s'

HOWBUY_RD_PROFUDUCT_PERMOISSIONS = "%s%s/admin/fof/pool/products/fund?access_token=%s&type=%s&org=%s"

##########################################################################
DATA_GETTING_TIPS = '[Getting data:]'
DATA_GETTING_FLAG = '#'
DATA_ROWS_TIPS = '%s rows data found.Please wait for a moment.'
DATA_INPUT_ERROR_MSG = 'date input error.'
NETWORK_URL_ERROR_MSG = '获取失败，请检查网络和URL'
DATE_CHK_MSG = '年度输入错误：请输入1989年以后的年份数字，格式：YYYY'
DATE_CHK_Q_MSG = '季度输入错误：请输入1、2、3或4数字'
TOP_PARAS_MSG = 'top有误，请输入整数或all.'
LHB_MSG = '周期输入有误，请输入数字5、10、30或60'

OFT_MSG = u'开放型基金类型输入有误，请输入all、equity、mix、bond、monetary、qdii'

DICT_NAV_EQUITY = {
    'fbrq': 'date',
    'jjjz': 'value',
    'ljjz': 'total',
    'change': 'change'
}

DICT_NAV_MONETARY = {
    'fbrq': 'date',
    'nhsyl': 'value',
    'dwsy': 'total',
    'change': 'change'
}

import sys
PY3 = (sys.version_info[0] >= 3)


def _write_head():
    sys.stdout.write(DATA_GETTING_TIPS)
    sys.stdout.flush()


def _write_console():
    sys.stdout.write(DATA_GETTING_FLAG)
    sys.stdout.flush()


def _write_tips(tip):
    sys.stdout.write(DATA_ROWS_TIPS % tip)
    sys.stdout.flush()


def _write_msg(msg):
    sys.stdout.write(msg)
    sys.stdout.flush()


def _check_input(year, quarter):
    if isinstance(year, str) or year < 1989:
        raise TypeError(DATE_CHK_MSG)
    elif quarter is None or isinstance(quarter, str) or quarter not in [1, 2, 3, 4]:
        raise TypeError(DATE_CHK_Q_MSG)
    else:
        return True



"""
超额收益观测体系
"""
import hbshare as hbs
import pandas as pd
import numpy as np
import datetime
from scipy import stats
import plotly
import plotly.graph_objs as go
from plotly.offline import plot as plot_ly
import plotly.figure_factory as ff


def plot_render(plot_dic, width=1200, height=600, **kwargs):
    kwargs['output_type'] = 'div'
    plot_str = plotly.offline.plot(plot_dic, **kwargs)
    print('%%angular <div style="height: %ipx; width: %spx"> %s </div>' % (height, width, plot_str))


class AlphaMonitor:
    def __init__(self, trade_date, benchmark_id='000905'):
        self.trade_date = trade_date
        self.benchmark_id = benchmark_id
        self._load_data()

    @staticmethod
    def _load_shift_date(date):
        trade_dt = datetime.datetime.strptime(date, '%Y%m%d')
        pre_date = (trade_dt - datetime.timedelta(days=100)).strftime('%Y%m%d')

        sql_script = "SELECT JYRQ, SFJJ, SFZM, SFYM FROM funddb.JYRL WHERE JYRQ >= {} and JYRQ <= {}".format(
            pre_date, date)
        res = hbs.db_data_query('readonly', sql_script, page_size=5000)
        df = pd.DataFrame(res['data']).rename(
            columns={"JYRQ": 'calendarDate', "SFJJ": 'isOpen',
                     "SFZM": "isWeekEnd", "SFYM": "isMonthEnd"}).sort_values(by='calendarDate')
        df['isOpen'] = df['isOpen'].astype(int).replace({0: 1, 1: 0})
        df['isWeekEnd'] = df['isWeekEnd'].fillna(0).astype(int)
        df['isMonthEnd'] = df['isMonthEnd'].fillna(0).astype(int)

        trading_day_list = df[df['isMonthEnd'] == 1]['calendarDate'].tolist()

        return trading_day_list[-1]

    @staticmethod
    def _load_benchmark_components(date):
        sql_script = "SELECT * FROM hsjy_gg.SecuMain where SecuCategory = 4 and " \
                     "SecuCode in ('000300', '000905', '000852')"
        res = hbs.db_data_query('readonly', sql_script)
        index_info = pd.DataFrame(res['data'])
        inner_code_series = index_info.set_index('SECUCODE')['INNERCODE']

        weight = []
        for benchmark_id in ['000300', '000905', '000852']:
            inner_code = inner_code_series.loc[benchmark_id]
            sql_script = "SELECT (select a.SecuCode from hsjy_gg.SecuMain a where a.InnerCode = b.InnerCode and " \
                         "rownum = 1) SecuCode, b.EndDate, b.Weight FROM hsjy_gg.LC_IndexComponentsWeight b WHERE " \
                         "b.IndexCode = '{}' and b.EndDate = to_date('{}', 'yyyymmdd')".format(inner_code, date)
            data = pd.DataFrame(hbs.db_data_query('readonly', sql_script)['data'])
            weight_df = data.rename(
                columns={"SECUCODE": "ticker", "ENDDATE": "effDate", "WEIGHT": "weight"})
            weight_df['benchmark_id'] = benchmark_id
            weight.append(weight_df[['ticker', 'benchmark_id']])

        return pd.concat(weight)

    def _load_data(self):
        # 个股收益
        sql_script = "SELECT SYMBOL, SNAME, VOTURNOVER, PCHG FROM finchina.CHDQUOTE WHERE" \
                     " TDATE = {}".format(self.trade_date)
        data = pd.DataFrame(hbs.db_data_query('readonly', sql_script, page_size=5000)['data'])
        data = data[data['SYMBOL'].str[0].isin(['0', '3', '6'])]
        data = data[data['VOTURNOVER'] > 1e-8]
        data = data[~data['SNAME'].str.contains('ST')]
        data = data[~data['SNAME'].str.contains('N')]
        data = data[~data['SNAME'].str.contains('C')]
        market_df = data.rename(
            columns={"SYMBOL": "ticker", 'PCHG': "return"})[['ticker', 'return']].dropna()
        # 指数收益
        trade_dt = datetime.datetime.strptime(self.trade_date, '%Y%m%d')
        pre_date = (trade_dt - datetime.timedelta(days=30)).strftime('%Y%m%d')
        sql_script = "SELECT JYRQ as TRADEDATE, SPJG as TCLOSE from funddb.ZSJY WHERE ZQDM = '{}' " \
                     "and JYRQ >= {} and JYRQ <= {}".format(self.benchmark_id, pre_date, self.trade_date)
        data = pd.DataFrame(hbs.db_data_query('readonly', sql_script)['data'])
        data['index_return'] = data['TCLOSE'].pct_change()
        index_return = data.set_index('TRADEDATE').loc[self.trade_date, 'index_return']
        # 指数成分数据
        shift_date = self._load_shift_date(self.trade_date)
        benchmark_cp = self._load_benchmark_components(shift_date)
        market_df = pd.merge(market_df, benchmark_cp, on='ticker', how='left')
        market_df['benchmark_id'].fillna('other', inplace=True)
        # 风格收益数据
        sql_script = "SELECT * FROM st_ashare.r_st_barra_factor_return where " \
                     "TRADE_DATE >= '{}' and TRADE_DATE <= '{}'".format(pre_date, self.trade_date)
        res = hbs.db_data_query('alluser', sql_script)
        style_df = pd.DataFrame(res['data'])
        style_df = pd.pivot_table(
            style_df, index='trade_date', columns='factor_name', values='factor_ret').sort_index()
        style_df = style_df[['size', 'btop', 'growth']]
        # 申万二级行业数据
        trade_dt = datetime.datetime.strptime(self.trade_date, '%Y%m%d')
        pre_date = (trade_dt - datetime.timedelta(days=10)).strftime('%Y%m%d')
        sql_script = "SELECT * FROM st_market.t_st_zs_hyzsdmdyb where fljb = {} and hyhfbz = 3 and sfyx = 1".format('2')
        res = hbs.db_data_query('alluser', sql_script, page_size=5000)
        data = pd.DataFrame(res['data']).rename(columns={"zsdm": "SYMBOL", "flmc": "INDEXSNAME"})
        map_dict = data.set_index('SYMBOL')['INDEXSNAME'].to_dict()
        industry_index = []
        for key, value in map_dict.items():
            sql_script = "SELECT JYRQ as TRADEDATE, ZQMC as INDEXNAME, SPJG as TCLOSE, " \
                         "LTSZ as NEG_MKV FROM funddb.ZSJY WHERE " \
                         "ZQDM = '{}' and JYRQ >= {} and " \
                         "JYRQ <= {}".format(key, pre_date, self.trade_date)
            res = hbs.db_data_query('readonly', sql_script, page_size=5000)
            df = pd.DataFrame(res['data'])
            df['INDEXNAME'] = value
            industry_index.append(df)
        industry_index = pd.concat(industry_index)
        industry_index = pd.pivot_table(
            industry_index, index='TRADEDATE', columns='INDEXNAME', values='TCLOSE').sort_index()
        industry_return = industry_index.pct_change().dropna().loc[self.trade_date].sort_values()

        self.data_param = {
            "market_info": market_df,
            "index_return": index_return,
            "style_factor": style_df,
            "industry_return": industry_return
        }

    def _return_structure_plot(self, market_df, index_return):
        market_df = market_df[market_df['return'].abs() <= 10]
        # bins
        interval_range = pd.interval_range(-10, 10, 100, closed='left')
        count_df = pd.cut(market_df['return'], bins=interval_range).value_counts().sort_index().to_frame('num')
        count_df['num_cumsum'] = count_df['num'].cumsum()
        count_df['start'] = count_df.index
        count_df['start'] = count_df['start'].apply(lambda x: round(x.left, 1))
        count_df['color'] = '#55CBF2'
        # locate the index return
        index_return = index_return * 100.
        lower = round(np.floor(index_return / 0.2) * 0.2, 2)
        count_df.loc[count_df['start'] == lower, 'color'] = '#ff2d51'
        # quantile
        win_ratio = round(100 - stats.percentileofscore(np.array(market_df['return']), index_return), 1)
        win_med = market_df[market_df['return'] >= index_return]['return'].median()
        lose_med = market_df[market_df['return'] < index_return]['return'].median()
        # locate the lower med
        lower = round(np.floor(lose_med / 0.2) * 0.2, 2)
        count_df.loc[count_df['start'] == lower, 'color'] = '#ffa400'
        # # locate the upper med
        lower = round(np.floor(win_med / 0.2) * 0.2, 2)
        count_df.loc[count_df['start'] == lower, 'color'] = '#ffa400'

        color_list = count_df[count_df['num_cumsum'] > 0]['color'].tolist()

        # text
        map_dict = {"000905": "中证500", "000852": "中证1000", "000300": "沪深300"}
        title_text = "【{}】{}收益率{}%, 跑赢指数的个股比例为{}%, 跑赢的中位数为{}%, 跑输的中位数为{}%".format(
            self.trade_date, map_dict[self.benchmark_id], round(index_return, 2),
            win_ratio, round(win_med, 2), round(lose_med, 2))

        layout = go.Layout(xaxis=dict(range=[-10, 10]), title=title_text, width=1200, height=600)
        data = go.Histogram(x=np.array(market_df['return']), xbins=dict(start=-10, end=10, size=0.2),
                            histfunc='count', histnorm='probability',
                            marker=dict(color=color_list, line=dict(color='white', width=1)))

        # figure = go.Figure(data=[data], layout=layout)
        # plot_ly(figure, filename='D:\\临时存储\\市场收益结构.html', auto_open=False)

        return data, layout

    @staticmethod
    def _market_skew_plot(market_df):
        group_labels = ['沪深300', '中证500', '中证1000']
        market_df = market_df[market_df['return'].abs() <= 10]
        data_list = [
            market_df[market_df['benchmark_id'] == x]['return'].tolist() for x in ['000300', '000905', '000852']]
        fig = ff.create_distplot(data_list,
                                 group_labels,
                                 show_hist=False,
                                 show_rug=False)

        fig.update_layout(title="宽基指数成分股收益率偏度",
                          xaxis_title='收益率(%)',
                          yaxis_title='density',
                          width=1200, height=600,
                          template='plotly_white',
                          showlegend=True,
                          legend=dict(
                              orientation="v",
                              y=1,
                              yanchor="top",
                              x=1.0,
                              xanchor="right"))

        # plot_ly(fig, filename='D:\\临时存储\\宽基指数偏度.html', auto_open=False)

        return fig.data, fig.layout

    @staticmethod
    def plotly_line(df, title_text, figsize=(1200, 600)):
        fig_width, fig_height = figsize
        data = []
        for col in df.columns:
            trace = go.Scatter(
                x=df.index.tolist(),
                y=df[col],
                name=col,
                mode="lines+markers",
                line=dict(
                    shape='spline'
                )
            )
            data.append(trace)

        tick_vals = df.index.tolist()[::3]

        layout = go.Layout(
            title=dict(text=title_text),
            autosize=False, width=fig_width, height=fig_height,
            yaxis=dict(tickfont=dict(size=12), tickformat=',.2%', showgrid=True),
            xaxis=dict(showgrid=True, tickvals=tick_vals),
            template='plotly_white'
        )

        # figure = go.Figure(data=data, layout=layout)
        # plot_ly(figure, filename='D:\\临时存储\\风格因子收益.html', auto_open=False)

        return data, layout

    @staticmethod
    def _industry_return_plot(df, index_return, title_text, figsize=(1200, 600)):
        fig_width, fig_height = figsize
        trace = go.Bar(
            x=df.index.tolist(),
            y=df.round(4).values.tolist(),
            name='申万二级行业收益',
            marker=dict(color='rgb(60, 127, 175)')
        )
        data = [trace]
        layout = go.Layout(
            title=dict(text=title_text),
            autosize=False, width=fig_width, height=fig_height,
            yaxis=dict(tickfont=dict(size=12),  tickformat=',.2%', showgrid=True),
            xaxis=dict(showgrid=True, showticklabels=False),
            template='plotly_white',
            legend=dict(orientation="h"),
            shapes=[
                {"type": "line",
                 "xref": "paper",
                 "x0": 0,
                 "x1": 1,
                 "name": "中证500指数收益",
                 "y0": index_return,
                 "y1": index_return,
                 "line": {'color': 'rgb(216, 0, 18)', 'width': 2, 'dash': 'dash'}}]
        )

        # figure = go.Figure(data=data, layout=layout)
        # plot_ly(figure, filename='D:\\临时存储\\行业收益分布.html', auto_open=False)

        return data, layout

    def run(self):
        data_param = self.data_param

        market_df, index_return = data_param['market_info'], data_param['index_return']
        data, layout = self._return_structure_plot(market_df, index_return)
        plot_render({"data": data, "layout": layout}, width=1200, height=600)

        data, layout = self._market_skew_plot(market_df)
        plot_render({"data": data, "layout": layout}, width=1200, height=600)

        style_df = data_param['style_factor'].cumsum()
        style_df.columns = [x.upper() for x in style_df.columns]
        data, layout = self.plotly_line(style_df, title_text="市场风格走势")
        plot_render({"data": data, "layout": layout}, width=1200, height=600)

        industry_return = data_param['industry_return']
        abs_ratio = industry_return.gt(0).sum() / industry_return.shape[0]
        relative_ratio = industry_return.gt(index_return).sum() / industry_return.shape[0]
        title = "申万二级行业收益：当日绝对上涨比例: {}%, 相对中证500胜率: {}%".format(
            np.round(100*abs_ratio, 2), np.round(100*relative_ratio))
        data, layout = self._industry_return_plot(industry_return, index_return, title)
        plot_render({"data": data, "layout": layout}, width=1200, height=600)


if __name__ == '__main__':
    AlphaMonitor('20220427').run()